
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnADOParams;

{$I Un.INC}

interface

 uses Classes, Db, ADODB_TLB, UnCustomDataSet, DBTables
{$IFDEF Un_D6}
     , Variants
{$ENDIF}
 ;

 Type

  TDataType = TFieldType;
  TParameterAttribute = (paSigned, paNullable, paLong);
  TParameterAttributes = set of TParameterAttribute;

  TUnADOParameter = class(TObject)
  private
   FParameter: _Parameter;
   function GetAttributes: TParameterAttributes;
   function GetDataType: TDataType;
   function GetName: string;
   function GetNumericScale: Byte;
   function GetParameterDirection: TParamType;
   function GetPrecision: Byte;
   function GetSize: Integer;
   function GetValue: Variant;
   procedure SetValue(Value: Variant);
   function GetParametr: _Parameter;
  protected
   function IsEqual(Value: TUnADOParameter): Boolean;
  public
   constructor Create(const AName: string; ADataType: TDataType; ADirection: TParamType; ASize: Integer; AValue: Variant);
   destructor Destroy; override;
   procedure LoadFromStream(Stream: TStream);
   property Parametr: _Parameter read GetParametr;
   property Name: string read GetName;
   property Attributes: TParameterAttributes read GetAttributes;
   property DataType: TDataType read GetDataType;
   property Direction: TParamType read GetParameterDirection;
   property NumericScale: Byte read GetNumericScale;
   property Precision: Byte read GetPrecision;
   property Size: Integer read GetSize;
   property Value: Variant read GetValue write SetValue;
  end;

  TUnADOParams = class(TObject)
   private
    FList: TList;
   protected
    function GetParams(Index: Integer): TUnADOParameter;
    function GetParamCount: Integer;
   public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function Add(const AName: string; ADataType: TDataType; ADirection: TParamType; ASize: Integer; AValue: Variant):TUnADOParameter;
    procedure Delete(Index: Integer);
    function Remove(Par: TUnADOParameter): boolean;
    property Params[Index: Integer]: TUnADOParameter read GetParams;
    property ParamCount: Integer read GetParamCount;
  end;

 var
  Str_InvalidEnumValue: string = 'Invalid Enum Value';

const
  ParameterAttributeValues: array[TParameterAttribute] of Integer =
    (adParamSigned, adParamNullable, adParamLong);

  ParameterDirectionValues: array[TParamType] of Integer =
    (adParamUnknown, adParamInput, adParamOutput, adParamInputOutput,
     adParamReturnValue);

  DataTypeValues: array[TDataType] of Integer =
   (
    adEmpty, adVarChar, adSmallint, adInteger, adUnsignedSmallint,
    adBoolean, adDouble, adDouble, adCurrency, adDate, adDate,
    adDate, adBinary, adVarBinary, adInteger, adLongVarBinary,
    adLongVarChar, adLongVarBinary, adLongVarBinary, adLongVarBinary,
    adLongVarBinary, adLongVarBinary, adEmpty
{$IFDEF UN_D4}
   , adChar, adVarWChar, adBigInt, adEmpty, adEmpty, adEmpty, adEmpty
{$ENDIF}
{$IFDEF UN_D5}
  , adVarBinary, adEmpty, adVariant, adIUnknown, adIDispatch, adGUID
{$ENDIF}
{$IFDEF UN_D6}
, adEmpty, adEmpty
{$ENDIF}
   );
//         ... ,  Oracle.

implementation

uses AdoClasses;

function OleEnumToOrd(OleEnumArray: array of Integer; Value: Integer): Cardinal;
begin
  for Result := Low(OleEnumArray) to High(OleEnumArray) do
    if Value = OleEnumArray[Result] then Exit;
  raise TUnDbException.Create(Str_InvalidEnumValue);
end;

{------------------------------------TUnADOParameter---------------------------}

constructor TUnADOParameter.Create(const AName: string; ADataType: TDataType; ADirection: TParamType; ASize: Integer; AValue: Variant);
begin
 inherited Create;
 FParameter:=CreateADOObject(Class_Parameter) as _Parameter;
 FParameter.Name:=AName;
 case ADataType of
  ftBytes, ftVarBytes, ftBlob, ftGraphic, ftParadoxOle, ftDBaseOle, ftTypedBinary: begin
    FParameter.Value:=AValue;
   end;
  ftMemo, ftFmtMemo: begin
    FParameter.Direction:=ParameterDirectionValues[ADirection];
    FParameter.Type_:=DataTypeValues[ADataType];
    FParameter.Size:=ASize;
    FParameter.Value:=AValue;
   end;
  else begin
    FParameter.Direction:=ParameterDirectionValues[ADirection];
    FParameter.Type_:=DataTypeValues[ADataType];
    FParameter.Size:=ASize;
    FParameter.Value:=AValue;
   end;
 end;
end;

destructor TUnADOParameter.Destroy;
begin
 FParameter:=nil;
 inherited;
end;

function TUnADOParameter.GetAttributes: TParameterAttributes;
var
  Attributes: Integer;
  Pa: TParameterAttribute;
begin
 Result:=[];
 Attributes:=FParameter.Attributes;
 if Attributes <> 0 then
  for Pa:=Low(TParameterAttribute) to High(TParameterAttribute) do
   if (ParameterAttributeValues[Pa] and Attributes) <> 0 then
    Include(Result, Pa);
end;

function TUnADOParameter.GetDataType: TDataType;
begin
  Result:=ADOTypeToFieldType(FParameter.Type_);
end;

function TUnADOParameter.GetName: string;
begin
 Result:=FParameter.Name;
end;

function TUnADOParameter.GetNumericScale: Byte;
begin
 Result:=FParameter.NumericScale;
end;

function TUnADOParameter.GetParameterDirection: TParamType;
begin
 Result:=TParamType(OleEnumToOrd(ParameterDirectionValues,
  FParameter.Direction));
end;

function TUnADOParameter.GetPrecision: Byte;
begin
 Result:=FParameter.Precision;
end;

function TUnADOParameter.GetSize: Integer;
begin
 Result:=FParameter.Size;
end;

function TUnADOParameter.GetValue: Variant;
begin
 Result:=FParameter.Value;
end;

procedure TUnADOParameter.SetValue(Value: Variant);
begin
 FParameter.Value:=Value;
end;

function TUnADOParameter.GetParametr: _Parameter;
begin
 Result:=FParameter;
end;

function TUnADOParameter.IsEqual(Value: TUnADOParameter): Boolean;
begin
  Result := (VarType(Self.Value) = VarType(Value.Value)) and
    (Name = Value.Name) and
    (DataType = Value.DataType) and (Direction = Value.Direction) and
    (NumericScale = Value.NumericScale) and (Precision = Value.Precision) and
    (Size = Value.Size);
end;

procedure TUnADOParameter.LoadFromStream(Stream: TStream);
var
  BinData: OleVariant;
  DataPtr: Pointer;
  Len: Integer;
begin
 with Stream do
  begin
   Position := 0;
   Len := Size;
   FParameter.Size:=Len;
   BinData := VarArrayCreate([0, Len-1], varByte);
   DataPtr := VarArrayLock(BinData);
   try
    ReadBuffer(DataPtr^, Len);
    Self.Value := BinData;
   finally
    VarArrayUnlock(BinData);
   end;
 end;
end;

{-------------------------------------TADOParams-------------------------------}

constructor TUnADOParams.Create;
begin
 inherited;
 FList:=TList.Create;
end;

destructor TUnADOParams.Destroy;
begin
 Clear;
 FList.Free;
 inherited;
end;

function TUnADOParams.GetParams(Index: Integer): TUnADOParameter;
begin
 Result:=FList[Index];
end;

function TUnADOParams.GetParamCount: Integer;
begin
 Result:=FList.Count;
end;

procedure TUnADOParams.Clear;
var I: Integer;
begin
 for I:=0 to FList.Count-1 do
  TObject(FList[I]).Free;
 FList.Clear;
end;

function TUnADOParams.Add(const AName: string; ADataType: TDataType; ADirection: TParamType; ASize: Integer; AValue: Variant):TUnADOParameter;
begin
 Result:=TUnADOParameter.Create(AName, ADataType, ADirection, ASize, AValue);
 FList.Add(Result);
end;

procedure TUnADOParams.Delete(Index: Integer);
begin
 TObject(FList[Index]).Free;
 FList.Delete(Index);
end;

function TUnADOParams.Remove(Par: TUnADOParameter): boolean;
begin
 Result:=FList.Remove(Par) > -1;
 Par.Free;
end;

end.
