
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnADOFetcher;

{$I Un.INC}

interface

 uses SysUtils, Classes, Db, UnCustomDataSet, UnSQLUpdater, UnADOParams,
      UnRecordSet, ADODB_TLB, AdoClasses, dbtables, UnFileServices
{$IFDEF Un_D6}
     , Variants
{$ENDIF}
      ;

 Type

  TADOBlobUpdater = class;

  TUnCustomADOCOmmand = class(TComponent)
   private
    FCommand: _Command;
    FSQL: TStringList;
    FRealSQL: string;
    FRecsAffected: OleVariant;
    FADOParams: TUnADOParams;
    FConnection: TUnAdoConnect;
    FOnGetParams: TUnGetParametrValue;
    function GetSQL: TStrings;
    procedure SetSQL(Value: TStrings);
    function GetRecsAffected: Integer;
    function GetConnection: TUnAdoConnect;
    procedure SetConnection(Value: TUnAdoConnect);
   protected
    procedure CopyParams(var ParamList: OleVariant); virtual;
    procedure CreateParams(var ParamList: OleVariant); virtual;
    property OnGetParams: TUnGetParametrValue read FOnGetParams write FOnGetParams;
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: _RecordSet; virtual;
    property SQL: TStrings read GetSQL write SetSQL;
    property RecsAffected: Integer read GetRecsAffected;
    property Connection: TUnAdoConnect read GetConnection write SetConnection;
    property Command: _Command read FCommand;
  end;

  TUnAdoCommand = class(TUnCustomADOCOmmand)
   private
    FParams: TParams;
   protected
    procedure SetParams(Value: TParams);
    procedure CopyParams(var ParamList: OleVariant); override;
    procedure DoGetParamValue(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant);
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: _RecordSet; override;
   published
    property Connection;
    property SQL;
    property Params: TParams read FParams write SetParams;
  end;

  TUnADOBlobRecordSet = class(TUnCustomAdoRecordSet)
   public
    property OnGetParam;
   end;

  TADOBlobUpdater = class(TSQLBlobUpdater)
   private
    FCommand: TUnCustomADOCOmmand;        // Update
    FRecordSet: TUnADOBlobRecordSet;          // Select
    procedure DoGetParamValue(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant);
   protected
    function GetGetSQL: TStrings;override;
    procedure SetGetSQL(Value: TStrings); override;
    function GetSetSQL: TStrings; override;
    procedure SetSetSQL(Value: TStrings); override;

    function GetConnection: TUnAdoConnect;
    procedure SetConnection(Value: TUnAdoConnect);
    function GetLockType: TUnAdoLockType;
    procedure SetLockType(Value: TUnAdoLockType);
    property Connection: TUnAdoConnect read GetConnection write SetConnection;
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
   published
    property LockType: TUnAdoLockType read GetLockType write SetLockType;
  end;

  TUnADOBlobItem = class(TUnSQLBlobItem)
   private
   protected
    function GetConnection: TUnAdoConnect;
    procedure SetConnection(Value: TUnAdoConnect);
    function GetLockType: TUnAdoLockType;
    procedure SetLockType(Value: TUnAdoLockType);
    function CreateBlobUpdater: TBlobUpdater; override;
   public
    procedure Assign(Source: TPersistent); override;
    property Connection: TUnAdoConnect read GetConnection write SetConnection;
   published
    property LockType: TUnAdoLockType read GetLockType write SetLockType;
  end;

  TUnADOFetcher = class(TUnCustomAdoFetcher)
   private
    FUpdateCommand: TUnCustomADOCOmmand;          // Update
    FDeleteCommand: TUnCustomADOCOmmand;          // Delete
    FInsertCommand: TUnCustomADOCOmmand;          // Insert
    FLockCommand: TUnCustomADOCOmmand;            // Lock
    FRefreshRecordSet: TUnADOBlobRecordSet;           // Refresh
    FListBlobs: TList;
    FReadConnection: TUnAdoConnect;
    procedure CreateExecProc(SQL: TStrings; Command: TUnCustomADOCOmmand);
    function BlobByFieldName(const FieldName: string): TADOBlobUpdater;
    procedure DoGetParamValue(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant);
   protected
    procedure SetReadConnect(Value: TUnAdoConnect); override;
    function GetWriteConnect: TUnAdoConnect; override;
    procedure SetWriteConnect(Value: TUnAdoConnect); override;
    function GetRefreshSQL: TStrings; override;
    procedure SetRefreshSQL(Value: TStrings); override;
    function GetLockSQL: TStrings; override;
    procedure SetLockSQL(Value: TStrings); override;
    procedure DoGetBlob(Field: TBlobField; var Data: TStream); override;
    procedure DoSetBlob(Field: TBlobField; var Data: TStream); override;
    function GetNewBlob: TBlobUpdater; override;
    function CreateCollection: TUnUpdateCollection;  override;
    procedure SetDataSet(Value: TUnDataSet); override;
    function GetLockType: TUnAdoLockType;
    procedure SetLockType(Value: TUnAdoLockType);
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DoInsert; override;
    procedure DoUpdate; override;
    procedure DoDelete; override;
    procedure DoRefresh;  override;
    procedure DoLock;  override;
   published
    property LockType: TUnAdoLockType read GetLockType write SetLockType;
  end;

implementation

 uses UnMessages;

{----------------------------------TUnADOCOmmand---------------------------------}

constructor TUnCustomADOCOmmand.Create(AOwner: TComponent);
begin
 inherited;
 FRecsAffected:=0;
 FSQL:=TStringList.Create;
 FADOParams:=TUnADOParams.Create;
 FCommand:=CreateADOObject(CLASS_Command) as _Command;
end;

destructor TUnCustomADOCOmmand.Destroy;
begin
 FADOParams.Free;
 FCommand:=nil;
 FSQL.Free;
 inherited;
end;

procedure TUnCustomADOCOmmand.CopyParams(var ParamList: OleVariant); 
begin
end;

function TUnCustomADOCOmmand.GetRecsAffected: Integer;
begin
 Result:=FRecsAffected;
end;

function TUnCustomADOCOmmand.GetConnection: TUnAdoConnect;
begin
 Result:=FConnection;
end;

procedure TUnCustomADOCOmmand.SetConnection(Value: TUnAdoConnect);
begin
 FConnection:=Value;
end;

procedure TUnCustomADOCOmmand.CreateParams(var ParamList: OleVariant);
var I: Integer;
begin
 if FADOParams.ParamCount = 0 then
  ParamList:=Null
 else
  begin
   ParamList:=VarArrayCreate([0, FADOParams.ParamCount-1], varVariant);
   for I:=0 to FADOParams.ParamCount-1 do
    ParamList[I]:=FADOParams.Params[I].Parametr;
  end;
end;                  

function TUnCustomADOCOmmand.Execute: _RecordSet;
var ParamsList: OleVariant;
    IsExec: boolean;
    I: Integer;
    TempSql: TStringList;
    
procedure CreateExec;
begin
 FCommand.ActiveConnection:=FConnection.Connection;
 ParsingSQL(TempSql.Text, FRealSQL, FADOParams, FOnGetParams, IsExec);
 FCommand.CommandText:=FRealSQL;
 FCommand.CommandTimeout:=FConnection.CommandTimeout;
 if IsExec then
  begin
   FCommand.CommandType:=adCmdStoredProc;
   CopyParams(ParamsList);        
   FCommand.Execute(FRecsAffected, ParamsList, adCmdStoredProc{ and adExecuteNoRecords});
  end
 else
  begin
   CreateParams(ParamsList);
   Result:=FCommand.Execute(FRecsAffected, ParamsList, adCmdText);
  end;
end;

begin
 TempSql:=TStringList.Create;
 try
  while FCommand.Parameters.Count > 0 do
   FCommand.Parameters.Delete(0);
  for I:=0 to FSQL.Count - 1 do
   begin
    if Trim(FSQL[I]) = '^' then
     begin
      CreateExec;
      TempSql.Clear;
     end
    else
     TempSql.Add(FSQL[I]);
   end;
  if TempSql.Count > 0 then
   CreateExec;
 finally
  TempSql.Free;
 end;
end;

function TUnCustomADOCOmmand.GetSQL: TStrings;
begin
 Result:=FSQL;
end;

procedure TUnCustomADOCOmmand.SetSQL(Value: TStrings);
begin
 FSQL.Assign(Value);
end;

{-----------------------------------TADOBlobUpdater----------------------------}

constructor TADOBlobUpdater.Create(AOwner: TComponent);
begin
 inherited;
 FCommand:=TUnCustomADOCOmmand.Create(Self);
 FCommand.OnGetParams:=DoGetParamValue;
 FRecordSet:=TUnADOBlobRecordSet.Create(Self);
 FRecordSet.OnGetParam:=DoGetParamValue;
end;

destructor TADOBlobUpdater.Destroy;
begin
 FRecordSet.Free;
 FCommand.Free;
 inherited;
end;

procedure TADOBlobUpdater.DoGetParamValue(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant);
begin
 if Updater.DataSet = nil then
  raise Exception.Create(str_Not_Assigned_DataSet);
  _DoGetParamValue(Updater.DataSet, ParamName, IsOld, ParamType, ParamValue);
end;

function TADOBlobUpdater.GetGetSQL: TStrings;
begin
 Result:=FRecordSet.SQL;
end;

procedure TADOBlobUpdater.SetGetSQL(Value: TStrings);
begin
 FRecordSet.SQL.Assign(Value);
end;

function TADOBlobUpdater.GetSetSQL: TStrings;
begin
 Result:=FCommand.SQL;
end;

procedure TADOBlobUpdater.SetSetSQL(Value: TStrings);
begin
 FCommand.SQL.Assign(Value);
end;

function TADOBlobUpdater.GetConnection: TUnAdoConnect;
begin
 Result:=FRecordSet.Connection;
end;

procedure TADOBlobUpdater.SetConnection(Value: TUnAdoConnect);
begin
 FRecordSet.Connection:=Value;
 FCommand.Connection:=Value;
end;

function TADOBlobUpdater.GetLockType: TUnAdoLockType;
begin
 Result:=FRecordSet.LockType;
end;

procedure TADOBlobUpdater.SetLockType(Value: TUnAdoLockType);
begin
 FRecordSet.LockType:=Value;
end;

{-------------------------------TUnADOBlobItem---------------------------------}

function TUnADOBlobItem.GetConnection: TUnAdoConnect;
begin
 Result:=(FBlobUpdater as TADOBlobUpdater).Connection;
end;

procedure TUnADOBlobItem.SetConnection(Value: TUnAdoConnect);
begin
 (FBlobUpdater as TADOBlobUpdater).Connection:=Value;
end;

function TUnADOBlobItem.GetLockType: TUnAdoLockType;
begin
 Result:=(FBlobUpdater as TADOBlobUpdater).LockType;
end;

procedure TUnADOBlobItem.SetLockType(Value: TUnAdoLockType);
begin
 (FBlobUpdater as TADOBlobUpdater).LockType:=Value;
end;

function TUnADOBlobItem.CreateBlobUpdater: TBlobUpdater;
begin
 Result:=TADOBlobUpdater.Create((Collection as TUnUpdateCollection).Owner.Owner);
end;

procedure TUnADOBlobItem.Assign(Source: TPersistent);
begin
 if Source is TUnADOBlobItem then
  FBlobUpdater:=TUnADOBlobItem(Source).FBlobUpdater
 else
  Inherited Assign(Source);
end;

{-------------------------------TUnADOUpdater----------------------------------}

constructor TUnADOFetcher.Create(AOwner: TComponent);
begin
 inherited;
 FUpdateCommand:=TUnCustomADOCOmmand.Create(Self);
 FUpdateCommand.OnGetParams:=DoGetParamValue;
 FDeleteCommand:=TUnCustomADOCOmmand.Create(Self);
 FDeleteCommand.OnGetParams:=DoGetParamValue;
 FInsertCommand:=TUnCustomADOCOmmand.Create(Self);
 FInsertCommand.OnGetParams:=DoGetParamValue;
 FLockCommand:=TUnCustomADOCOmmand.Create(Self);
 FLockCommand.OnGetParams:=DoGetParamValue;
 FRefreshRecordSet:=TUnADOBlobRecordSet.Create(Self);
 FRefreshRecordSet.OnGetParam:=DoGetParamValue;
 FListBlobs:=TList.Create;
end;

destructor TUnADOFetcher.Destroy;
var I: Integer;
begin
 FUpdateCommand.Free;
 FDeleteCommand.Free;
 FInsertCommand.Free;
 FLockCommand.Free;
 FRefreshRecordSet.Free;
 for I:=0 to  FListBlobs.Count -1 do
  TObject(FListBlobs[I]).Free;
 inherited;
end;

procedure TUnADOFetcher.SetReadConnect(Value: TUnAdoConnect);
begin
 inherited;
 FReadConnection:=Value;
end;

function TUnADOFetcher.GetWriteConnect: TUnAdoConnect;
begin
 Result:=FUpdateCommand.Connection;
end;

procedure TUnADOFetcher.SetWriteConnect(Value: TUnAdoConnect);
begin
 FUpdateCommand.Connection:=Value;
 FDeleteCommand.Connection:=Value;
 FInsertCommand.Connection:=Value;
 FLockCommand.Connection:=Value;
 FRefreshRecordSet.Connection:=Value;
end;

procedure TUnADOFetcher.DoGetParamValue(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant);
begin
 if DataSet = nil then
  raise Exception.Create(str_Not_Assigned_DataSet);
 _DoGetParamValue(DataSet, ParamName, IsOld, ParamType, ParamValue);
end;

procedure TUnADOFetcher.SetDataSet(Value: TUnDataSet);
begin
 inherited;
end;

function TUnADOFetcher.CreateCollection: TUnUpdateCollection;
begin
 Result:=TUnUpdateCollection.Create(TUnADOBlobItem);
 Result.Owner:=Self;
end;

function TUnADOFetcher.GetLockType: TUnAdoLockType;
begin
 Result:=FRefreshRecordSet.LockType;
end;

procedure TUnADOFetcher.SetLockType(Value: TUnAdoLockType);
begin
 FRefreshRecordSet.LockType:=Value;
end;

function TUnADOFetcher.GetRefreshSQL: TStrings;
begin
 Result:=FRefreshRecordSet.SQL;
end;

procedure TUnADOFetcher.SetRefreshSQL(Value: TStrings);
begin
 FRefreshRecordSet.SQL.Assign(Value);
end;

function TUnADOFetcher.GetLockSQL: TStrings;
begin
 Result:=FLockCommand.SQL;
end;

procedure TUnADOFetcher.SetLockSQL(Value: TStrings);
begin
 FLockCommand.SQL.Assign(Value);
end;

function TUnADOFetcher.GetNewBlob: TBlobUpdater;
begin
 Result:=TADOBlobUpdater.Create(Self.Owner);
 Result.Updater:=Self;
end;

procedure TUnADOFetcher.DoInsert;
begin
 CreateExecProc(InsertSQL, FInsertCommand);
end;

procedure TUnADOFetcher.DoUpdate;
begin
 CreateExecProc(UpdateSQL, FUpdateCommand);
end;

procedure TUnADOFetcher.DoDelete;
begin
 CreateExecProc(DeleteSQL, FDeleteCommand);
end;

procedure TUnADOFetcher.DoRefresh;
var I, J: Integer;
    Fld: TField;
    Buffer: PChar;
begin
 try
  FRefreshRecordSet.Open;
  for I:=0 to DataSet.FieldCount-1 do
   begin //   
    Fld:=DataSet.Fields[I];
    if (Fld.FieldKind = fkData) then
     begin
      for J:=0 to FRefreshRecordSet.RecordSet.Fields.Count-1 do
       begin
        if CompareText(Fld.FieldName, FRefreshRecordSet.RecordSet.Fields[J].Name) = 0 then
         begin
          if Fld.IsBlob then
           GetMem(Buffer, DataSet.RecordSet.BufferSize)
          else
           GetMem(Buffer, Fld.DataSize);
          try
           AFechFuncList[Byte(Fld.DataType)](FRefreshRecordSet.RecordSet, J, Buffer);
           DataSet.SetFieldData(Fld, Buffer);
          finally
           FreeMem(Buffer);
          end;
          Break;
         end;
       end;
     end;
   end;
 finally
  FRefreshRecordSet.Close;
 end;
end;

procedure TUnADOFetcher.DoLock;
begin
 CreateExecProc(LockSQL, FLockCommand);
end;

function TUnADOFetcher.BlobByFieldName(const FieldName: string): TADOBlobUpdater;
var I: Integer;
begin
 Result:=nil;
 for I:=0 to BlobsCollection.Count-1 do
  begin
   if CompareText(TUnADOBlobItem(BlobsCollection.Items[I]).FBlobUpdater.FieldName, FieldName) = 0 then
    begin
     Result:=TUnADOBlobItem(BlobsCollection.Items[I]).FBlobUpdater as TADOBlobUpdater;
     Break;
    end;
  end;
end;

procedure TUnADOFetcher.DoGetBlob(Field: TBlobField; var Data: TStream);
var S: TADOBlobUpdater;
    I: Integer;
    D: PChar;
    V: OleVariant;
    Count: Integer;
    Str: string;
begin
 Data.Size:=0;
 S:=BlobByFieldName(Field.FieldName);
 if S <> nil then
  begin
   try
    if Field.DataSet.State = dsEdit then
     S.Connection:=WriteConnect
    else
     S.Connection:=ReadConnect;
    S.FRecordSet.Open;
     for I:=0 to S.FRecordSet.FieldCount-1 do
      begin
       if CompareText(S.FRecordSet.RecordSet.Fields[I].Name, Field.FieldName) = 0 then
        begin
          if not S.FRecordSet.RecordSet.EOF and
           (VarType(S.FRecordSet.RecordSet.Fields[I].Value) <> varNull) then
           begin
            V:=S.FRecordSet.RecordSet.Fields[I].Value;
            Data.Position:=0;
            case Field.DataType of
             ftFmtMemo, ftMemo: begin
               Str:=V;
               Data.Write(Str[1], Length(Str));
              end
             else begin
               Count:=VarArrayHighBound(V, 1)+1;
               D:=VarArrayLock(V);
               try
                Data.Write(D^, Count);
               finally
                VarArrayUnlock(V);
               end;
              end;
            end;
           end;
         Break;
        end;
      end;
     finally
      S.FRecordSet.Close;
     end;
  end;
end;

procedure TUnADOFetcher.DoSetBlob(Field: TBlobField; var Data: TStream);
var S: TADOBlobUpdater;
begin
 S:=BlobByFieldName(Field.FieldName);
 if S <> nil then
  begin
   S.Connection:=WriteConnect;
   CreateExecProc(S.SetSQL, S.FCommand);
  end;
end;

procedure TUnADOFetcher.CreateExecProc(SQL: TStrings; Command: TUnCustomADOCOmmand);
var I: Integer;
    TempSql: TStringList;
begin
 TempSql:=TStringList.Create;
 try
  for I:=0 to SQL.Count-1 do
   begin
    if Trim(SQL[I]) = '^' then
     begin
      Command.SQL.Assign(TempSql);
      Command.Execute;
      TempSql.Clear;
     end
    else
     TempSql.Add(SQL[I]);
   end;
  if TempSql.Count > 0 then
   begin
    Command.SQL.Assign(TempSql);
    Command.Execute;
   end;
 finally
  TempSql.Free;
 end;
end;

{----------------------------------TUnAdoCommand-------------------------------}

constructor TUnAdoCommand.Create(AOwner: TComponent);
begin
 inherited;
 FParams:=TParams.Create;
 OnGetParams:=DoGetParamValue;
end;

destructor TUnAdoCommand.Destroy;
begin
 FParams.Free;
 inherited;
end;

procedure TUnAdoCommand.CopyParams(var ParamList: OleVariant);
var I: Integer;
    ADOParam: _Parameter;
    V: OLEVariant;
    Buffer: PChar;
    TempStr: string;
    Len: Integer;
begin
 ParamList:=FCommand.Parameters;
 if FParams.Count > 0 then
  if (CompareText(FParams[0].Name, '@RETURN_VALUE') <> 0) and
     (CompareText(FParams[0].Name, 'RETURN_VALUE') <> 0) then
   begin
    ADOParam:=CreateADOObject(Class_Parameter) as _Parameter;
    ADOParam.Name:='@RETURN_VALUE';
    ADOParam.Direction:=adParamReturnValue;
    ADOParam.Type_:=DataTypeValues[ftInteger];
    ADOParam.Value:=0;
    FCommand.Parameters.Append(ADOParam);
   end;
 for I:=0 to FParams.Count-1 do
  begin
   if FParams[I].Name[1] <> '@' then
    FParams[I].Name:='@'+FParams[I].Name;
   ADOParam:=CreateADOObject(Class_Parameter) as _Parameter;
   ADOParam.Name:=FParams[I].Name;
   case FParams[I].DataType of
    ftBytes, ftVarBytes, ftBlob, ftGraphic, ftParadoxOle, ftDBaseOle, ftTypedBinary: begin
      TempStr:=FParams[I].AsBlob;
      Len:=Length(TempStr);
      V:=VarArrayCreate([0, Len], VarByte);
      Buffer:=VarArrayLock(V);
      try
       Move(TempStr[1], Buffer^, Len);
      finally
       VarArrayUnlock(V);
      end;
      ADOParam.Value:=V;
     end;
    ftMemo, ftFmtMemo: begin
      ADOParam.Direction:=ParameterDirectionValues[FParams[I].ParamType];
      ADOParam.Type_:=DataTypeValues[FParams[I].DataType];
      ADOParam.Size:=Length(FParams[I].AsBlob);
      ADOParam.Value:=FParams[I].AsBlob;
     end;
    ftString: begin
      ADOParam.Direction:=ParameterDirectionValues[FParams[I].ParamType];
      ADOParam.Type_:=DataTypeValues[FParams[I].DataType];
      ADOParam.Value:=FParams[I].AsString;
      ADOParam.Size:=Length(ADOParam.Value);
     end;
    ftSmallint, ftInteger, ftWord, ftAutoInc, ftBoolean,
    ftFloat, ftCurrency, ftBCD, ftDate, ftTime, ftDateTime: begin
      ADOParam.Direction:=ParameterDirectionValues[FParams[I].ParamType];
      ADOParam.Type_:=DataTypeValues[FParams[I].DataType];
      ADOParam.Value:=FParams[I].Value;
     end;
    else begin
      ADOParam.Direction:=ParameterDirectionValues[FParams[I].ParamType];
      ADOParam.Type_:=adVariant;
      ADOParam.Value:=FParams[I].Value;
     end;
   end;
   FCommand.Parameters.Append(ADOParam);
  end;
end;

function TUnAdoCommand.Execute: _RecordSet;
var I, J: Integer;
    Param: TParam;
    ADOParam: _Parameter;
    Str: TStream;
    V: Variant;
    D: PChar;
    Count: Integer;
begin
 Result:=inherited Execute;
 I:=0;
 while I < FParams.Count do
  begin
   Param:=FParams[I];
   if Param.ParamType in [ptOutput, ptInputOutput, ptResult] then
    begin
     for J:=0 to FCommand.Parameters.Count-1 do
      begin
       ADOParam:=FCommand.Parameters[J];
       if CompareText(Param.Name, ADOParam.Name) = 0 then
        begin
         case ADOTypeToFieldType(ADOParam.Type_) of
          ftBlob: begin
            V:=ADOParam.Value;
            Str:=GetTempStream;
            try
             Count:=VarArrayHighBound(V, 1)+1;
             D:=VarArrayLock(V);
             try
              Str.Write(D^, Count);
              Str.Position:=0;
              Param.LoadFromStream(Str, ftBlob);
             finally
              VarArrayUnlock(V);
             end;
            finally
             Str.Free;
            end;
           end;
          ftMemo, ftFmtMemo: begin
           Param.AsBlob:=ADOParam.Value;
           end
          else Param.Value:=ADOParam.Value;
         end;
         if Param.Name[1] = '@' then
          Param.Name:=Copy(Param.Name, 2, Length(Param.Name)-1);
         Break;
        end;
      end;
     Inc(I);
    end
   else if Param.Name[1] <> '@' then
    begin
     FParams.RemoveParam(Param);
     Param.Free;
    end
   else
    Inc(I);
  end;
end;

procedure TUnAdoCommand.DoGetParamValue(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant);
var I: Integer;
    Par: TParam;
    IsOk: boolean;
    TempString: string;
    Len: Integer;
    Buffer: PChar;
begin
 IsOk:=false;
 for I:=0 to FParams.Count-1 do
  begin
   Par:=FParams[I];
   if CompareText(Par.Name, ParamName) = 0 then
    begin
       ParamType:=Par.DataType;
       case Par.DataType of
        ftFmtMemo, ftMemo: ParamValue:=Par.AsMemo;
        ftBytes, ftVarBytes, ftBlob, ftGraphic, ftParadoxOle, ftDBaseOle, ftTypedBinary: begin
          TempString:=Par.AsBlob;
          Len:=Length(TempString);
          ParamValue:=VarArrayCreate([0, Len], varByte);
          Buffer:=VarArrayLock(ParamValue);
          try
           Move(TempString[1], Buffer^, Len);
          finally
           VarArrayUnlock(ParamValue);
          end;
         end;
        else ParamValue:=Par.Value;
       end;
     IsOk:=true;
     Break;
    end;
  end;
 if not IsOk then
  raise Exception.CreateFmt(str_Not_Found_Param, [ParamName]);
end;

procedure TUnAdoCommand.SetParams(Value: TParams);
begin
 FParams.Assign(Value);
end;

initialization

 RegisterClass(TADOBlobUpdater);

finalization

 UnRegisterClass(TADOBlobUpdater);

end.
