
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnCustomDataSet;

{$I Un.INC}

interface

  uses sysutils, Classes, Controls, Forms, Db,
{$IFDEF Un_D6}
       DesignIntf,
{$ELSE}
       DsgnIntf,
{$ENDIF}
   UnConstants, UnRecordSet;

  Type

   TAddData = record IsData : boolean; end;
   PAddData = ^TAddData;

   TUnDbException = class(Exception);

   TOffsetsArray = array[1..MaxListSize] of Integer;
   POffsetsArray = ^TOffsetsArray;

   TUnGetSetBlob = procedure(DataSet: TDataSet; Field: TBlobField; var Data: TStream) of Object;
   TUnDbNotifyEvent = procedure(DataSet: TDataSet) of Object;
   TCompareData = function(Buffer1, Buffer2: PChar): Integer of Object;
   TUnFieldCompare = function(ListFields: TList; P1, P2: Pointer; I1, I2: Integer): Integer of Object;
   TUnGetParametrValue = procedure(const ParamName: string; IsOld: boolean; var ParamType: TFieldType; var ParamValue: Variant) of Object;
   TFindFieldData = function (Buffer: Pointer; Field: TField): Pointer of Object;
   PDateTime = ^TDateTime;

   TUnDataSet = class;

   TUnCustomUpdater = class;
   TBlobUpdater = class;

   TUnBlobItem = class(TCollectionItem)
    private
     function GetUpdater: TUnCustomUpdater;
    protected
     FBlobUpdater: TBlobUpdater;
     function CreateBlobUpdater: TBlobUpdater; virtual; abstract;
     function GetFieldName: string;
     procedure SetFieldName(const Value: string);
    public
     constructor Create(Collection: TCollection); override;
     constructor CreateNew(Collection: TCollection; BlobUpdater: TBlobUpdater); virtual;
     destructor Destroy; override;
     property Updater: TUnCustomUpdater read GetUpdater;

    published
     property FieldName: string read GetFieldName write SetFieldName;
   end;

   TUnBlobItemClass = class of TUnBlobItem;

   TUnUpdateCollection = class(TCollection)
    private
     FOwner: TUnCustomUpdater;
    protected
     function  GetOwner: TPersistent; override;
     procedure DoAddItem(Item: TUnBlobItem);
     procedure DoDeleteItem(Item: TUnBlobItem);
    public
     property Owner: TUnCustomUpdater read FOwner write FOwner;
   end;

   TBlobUpdater = class(TComponent)
    private
     FFieldName: string;
     FUpdater: TUnCustomUpdater;
    protected
     procedure SetUpdater(Value: TUnCustomUpdater);
    public
    published
     destructor Destroy; override;
     property Updater: TUnCustomUpdater read FUpdater write SetUpdater;
     property FieldName: string read FFieldName write FFieldName;
   end;

   TUnCustomUpdater = class(TComponent)
    private
     FCollection: TUnUpdateCollection;
     FRefreshBeforeEdit: boolean;
     FRefreshAfterPost: boolean;
     FLockBeforeEdit: boolean;
     FRefreshBlobsOnEdit: boolean;
    protected
     procedure SetDataSet(Value: TUnDataSet); virtual; abstract;
     function GetDataSet: TUnDataSet; virtual; abstract;
     procedure DoGetBlob(Field: TBlobField; var Data: TStream); virtual; abstract;
     procedure DoSetBlob(Field: TBlobField; var Data: TStream); virtual; abstract;
     function GetNewBlob: TBlobUpdater; virtual; abstract;
     function CreateCollection: TUnUpdateCollection;  virtual; abstract;
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure DoInsert; virtual; abstract;
     procedure DoUpdate; virtual; abstract;
     procedure DoDelete; virtual; abstract;
     procedure DoRefresh;  virtual; abstract;
     procedure DoLock;  virtual; abstract;
     property DataSet: TUnDataSet read GetDataSet write SetDataSet;
    published
     property BlobsCollection: TUnUpdateCollection read FCollection;
     property RefreshBeforeEdit: boolean read FRefreshBeforeEdit write FRefreshBeforeEdit;
     property RefreshAfterPost: boolean read FRefreshAfterPost write FRefreshAfterPost;
     property LockBeforeEdit: boolean read FLockBeforeEdit write FLockBeforeEdit;
     property RefreshBlobsOnEdit: boolean read FRefreshBlobsOnEdit write FRefreshBlobsOnEdit;
   end;

   //   Blob
   TUnMemoryStream = class(TStream)
    private
     FDataSet: TUnDataSet;       //  DataSet 
     FField: TBlobField;         //  ,   
     FMode: TBlobStreamMode;     //   : Read, Write...
     FModified: boolean;         //   
     FBlob: Integer;
     FPosition: Integer;
    protected
    public
     constructor Create(Field: TBlobField; Mode: TBlobStreamMode);
     destructor Destroy; override;
     function Read(var Buffer; Count: Longint): Longint; override;
     function Write(const Buffer; Count: Longint): Longint; override;
     function Seek(Offset: Longint; Origin: Word): Longint; override;
     procedure Truncate; 
     procedure SetSize(NewSize: Longint); override;
     property Modified: boolean read FModified;
   end;

   TUnDataSet = class(TDataSet)
    private
     FRealRecordPos: Integer;       //   
     FListBlobs: TList;             //  
     FIsDeleted: boolean;           //    
     FOnGetBlob: TUnGetSetBlob;     //   
     FOnSetBlob: TUnGetSetBlob;     //     
     FGetFieldDef: TUnDbNotifyEvent;   //      
     FIsRecalck: boolean;              //  
     FCaseInsensitive: boolean;        //  
     FPartialKey: boolean;             //  
     FIsDescending: boolean;           //  
     FCompareFieldes: array[0..22] of TCompareData; //     
     FSortBy: string;       //   ";"  ,     
     FShowCursor: boolean;
     FCursor: TCursor;
     FRecordSetDispatcher: TUnCustomRecordSetDispatcher;
     FIsOpen: boolean;
     FOldRecNoForBlobs: Integer;
     FOpenFromStreamNow: boolean;
     FOldBuffer: PChar;
     FStopScroll: boolean;
     //FListCalckFields: TList;
     function CompareInteger(Buffer1, Buffer2: PChar): Integer;
     function CompareString(Buffer1, Buffer2: PChar): Integer;
     function CompareFloat(Buffer1, Buffer2: PChar): Integer;
     function CompareDateTime(Buffer1, Buffer2: PChar): Integer;
     function CompareBool(Buffer1, Buffer2: PChar): Integer;
     function CompareSmallInt(Buffer1, Buffer2: PChar): Integer;
     function CompareWord(Buffer1, Buffer2: PChar): Integer;
     function CompareError(Buffer1, Buffer2: PChar): Integer;
     procedure _CheckPositionCursor(CurPos: Integer);
     function _RecordFilter: Boolean;
     function _GetActiveRecBuf(var RecBuf: PChar): Boolean;
     function _FindFieldData(Buffer: Pointer; Field: TField): Pointer;
     procedure _CheckWriteMode;
     procedure _CreateListFields(const AIndexName: string; ListFields: TList; var IsCalck: boolean);
     procedure _QuickSort(AListFields: TList; SortList: TUnCustomRecordSet; L, R: Integer; SCompare: TUnFieldCompare);
     function _CompareFields(ListFields: TList; P1, P2: Pointer; I1, I2: Integer): Integer;
     function _QuickLocate(FieldLst: TList; const KeyValues: Variant; ACaseInsensitive, APartialKey: boolean): boolean;
     function _SlowlyLocate(FieldLst: TList; const KeyValues: Variant; ACaseInsensitive, APartialKey: boolean): boolean;
     function _CompareRecord(ListFields: TList; const KeyValues: Variant): Integer;
     procedure SetRecordPosition(AValue: Integer);
     procedure ClearBlobsStream;
    protected
     FRecordSet: TUnCustomRecordSet;
     FBufferSize: Word;                //    
     FCalckFieldSize: Integer;
     FRecordSize: Word;                //       
     FClearBuffer: PChar;              //     .     
     FIsRunAllFecher: boolean;         //        
     FBlobPresents: boolean;           // ,   
     FIsFechAll: boolean;              // ,   
     FListOffsets: POffsetsArray;      //      
     FUpdater: TUnCustomUpdater;       //   ,    ,     -    
     FBatchMode: boolean;
     FIsLoaded: boolean;
     FOnPostNow: boolean;
     FCalckPosition: Integer;
     FLoadFieldDefsFromStream: boolean;
     procedure _FechAll;
     procedure _CheckNextFech; virtual;
     procedure _OpenData; virtual;
     procedure _InternalAddFechData(Buffer: PChar);
     procedure Clear; virtual;
     function AllocRecordBuffer: PChar; override;
     procedure FreeRecordBuffer(var Buffer: PChar); override;
     function GetRecord(Buffer: PChar; GetMode: TGetMode; DoCheck: Boolean): TGetResult; override;
     procedure InternalInitRecord(Buffer: PChar); override;
     function GetRecordSize: Word; override;
     procedure InternalFirst; override;
     procedure InternalLast; override;
     procedure InternalAddRecord(Buffer: Pointer; Append: Boolean); override;
     procedure InternalDelete; override;
     procedure InternalPost; override;
     procedure GetBookmarkData(Buffer: PChar; Data: Pointer); override;
     procedure SetBookmarkData(Buffer: PChar; Data: Pointer); override;
     function GetBookmarkFlag(Buffer: PChar): TBookmarkFlag; override;
     procedure SetBookmarkFlag(Buffer: PChar; Value: TBookmarkFlag); override;
     procedure InternalGotoBookmark(Bookmark: Pointer); override;
     procedure InternalSetToRecord(Buffer: PChar); override;
     procedure InternalClose; override;
     procedure InternalOpen; override;
     procedure InternalHandleException; override;
     procedure InternalInitFieldDefs; override;
     procedure DoBeforeClose; override;
     function IsCursorOpen: Boolean; override;
     function GetRecordCount: Integer; override;
     function GetRecNo: Integer; override;
     procedure SetRecNo(Value: Integer); override;
     function GetIsLoaded: boolean; virtual;
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     property OnGetFieldDef: TUnDbNotifyEvent read FGetFieldDef write FGetFieldDef;
     procedure DoBeforeEdit; override;
     procedure DoAfterEdit; override;
     procedure _SetBlobsToServer;
     procedure SetDataToServer; virtual; abstract;
     procedure _CreateBuffers;
     procedure AlocFetchBuffer; virtual;
     procedure DoBeforeScroll; override;
     procedure DoAfterScroll; override;
     procedure ClearCalcFields(Buffer: PChar); override;
     property OnGetBlob: TUnGetSetBlob read FOnGetBlob write FOnGetBlob;
     property OnSetBlob: TUnGetSetBlob read FOnSetBlob write FOnSetBlob;
     property RecordSetDispetcher: TUnCustomRecordSetDispatcher read FRecordSetDispatcher write FRecordSetDispatcher;
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     function CompareBookmarks(Bookmark1, Bookmark2: TBookmark): Integer; override;
     function BookmarkValid(Bookmark: TBookmark): Boolean; override;
     function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream; override;
     function GetFieldData(Field: TField; Buffer: Pointer): Boolean; override;
     procedure SetFieldData(Field: TField; Buffer: Pointer); override;
     procedure SortOnFields(const FieldNames: string; ACaseInsensitive, APartialKey, ADescending: Boolean);
     function Locate(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean; override;
     procedure StartBatchMode; virtual;
     procedure UpdateBatch(IsAll: boolean); virtual;
     procedure CancelBatchMode;
     procedure OpenFromStream(Str: TStream);
     procedure Pack;
     procedure SaveToStream(Str: TStream);
     procedure Empty;
     procedure AddData(const Str: TStream);
     procedure SaveData(const Str: TStream);
     procedure FetchAll;
     property IsFechAll: boolean read FIsFechAll;
     property IsLoaded: boolean read GetIsLoaded;
     property IsDeleted: boolean read FIsDeleted;
     property SortBy: string read FSortBy;
     property CaseInsensitive: boolean read FCaseInsensitive;
     property PartialKey: boolean read FPartialKey;
     property IsDescending: boolean read FIsDescending;          
     property RecordSet: TUnCustomRecordSet read FRecordSet;
     property BatchMode: boolean read FBatchMode;
     property RecordPos: Integer read FRealRecordPos write SetRecordPosition;
    published
     property Cursor: TCursor read FCursor write FCursor;
     property ShowCursor: boolean read  FShowCursor write FShowCursor;
   end;

  TUnMemoryTable = class(TUnDataSet)
   protected
    procedure InternalInitFieldDefs; override;
   published
     // ,    ,   
     property Filtered;
     //property FieldDefs; 
     property RecordSetDispetcher;
     property AutoCalcFields;
     property OnGetBlob;
     property OnSetBlob;
     property AfterCancel;
     property AfterClose;
     property AfterDelete;
     property AfterEdit;
     property AfterInsert;
     property AfterOpen;
     property AfterPost;
     property AfterScroll;
     property BeforeCancel;
     property BeforeClose;
     property BeforeDelete;
     property BeforeEdit;
     property BeforeInsert;
     property BeforeOpen;
     property BeforePost;
     property BeforeScroll;
     property OnCalcFields;
     property OnDeleteError;
     property OnEditError;
     property OnNewRecord;
     property OnPostError;
     property OnFilterRecord;
     property Active;
   end;

 var

  //   
 str_No_Record :             string = 'No Record !';
 str_Not_Assign_Stream :     string = 'Assign data stream before open !';
 str_Not_Write_Mode :        string = 'The data in a stream are not written !';
 str_Not_Blod_To_Index :     string = 'It is impossible to index BLOB !';
 str_Not_Support :           string = 'Not support !';
 str_Not_Edit_Mode :         string = 'Dataset not in edit or insert mode';
 str_Not_For_Open_DataSet :  string = 'This operation is inapplicable to an open data set !';
 str_Alredy_Opened :         string = 'DataSet is already opened !';
 str_Not_Set_RecorsSetDisp : string = 'Set RecordSetDispetcher before open !';

implementation

 uses UnFileServices, UnStreamRecordSet;

{----------------------------------TUnBlobItem---------------------------------}

constructor TUnBlobItem.Create(Collection: TCollection);
begin
 inherited;
 (Collection as TUnUpdateCollection).DoAddItem(Self);
end;

constructor TUnBlobItem.CreateNew(Collection: TCollection; BlobUpdater: TBlobUpdater);
begin
 inherited Create(Collection);
 FBlobUpdater:=BlobUpdater;
 (Collection as TUnUpdateCollection).DoAddItem(Self);
end;

destructor TUnBlobItem.Destroy;
begin
 (Collection as TUnUpdateCollection).DoDeleteItem(Self);
 inherited;
end;

function TUnBlobItem.GetUpdater: TUnCustomUpdater;
begin
 Result:=(Collection as TUnUpdateCollection).FOwner;
end;

function TUnBlobItem.GetFieldName: string;
begin
 Result:=FBlobUpdater.FieldName;
end;

procedure TUnBlobItem.SetFieldName(const Value: string);
begin
 FBlobUpdater.FieldName:=Value;
end;

{---------------------------------TUnCollection--------------------------------}

function TUnUpdateCollection.GetOwner: TPersistent;
begin
 Result:=FOwner;
end;

procedure TUnUpdateCollection.DoAddItem(Item: TUnBlobItem);
begin
 if Item.FBlobUpdater = nil then
  begin
   Item.FBlobUpdater:=Item.CreateBlobUpdater;
   Item.FBlobUpdater.FUpdater:=FOwner;
  end;
end;

procedure TUnUpdateCollection.DoDeleteItem(Item: TUnBlobItem);
begin
 if Item.FBlobUpdater <> nil then
  begin
   if Item.FBlobUpdater.Owner <> nil then
    Item.FBlobUpdater.Owner.RemoveComponent(Item.FBlobUpdater);
   Item.FBlobUpdater.Free;
  end;
end;

{---------------------------------TBlobUpdater---------------------------------}

destructor TBlobUpdater.Destroy;
var I: Integer;
begin
 if FUpdater <> nil then
  begin
   for I:=0 to FUpdater.FCollection.Count-1 do
    begin
     if (FUpdater.FCollection.Items[I] as TUnBlobItem).FBlobUpdater = Self then
      begin
       (FUpdater.FCollection.Items[I] as TUnBlobItem).FBlobUpdater:=nil;
       Break;
      end;
    end;
  end;
 inherited;
end;

procedure TBlobUpdater.SetUpdater(Value: TUnCustomUpdater);
var I: Integer;
    IsOk: boolean;
begin
 FUpdater:=Value;
 if FUpdater <> nil then
  begin
   IsOk:=false;
   for I:=0 to FUpdater.FCollection.Count-1 do
    begin
     if (FUpdater.FCollection.Items[I] as TUnBlobItem).FBlobUpdater = Self then
      begin
       IsOk:=true;
       Break;
      end;
    end;
   if not IsOk then
    TUnBlobItemClass(FUpdater.FCollection.ItemClass).CreateNew(FUpdater.FCollection, Self);
  end;
end;

{---------------------------------TUnCustomUpdater-----------------------------}

constructor TUnCustomUpdater.Create(AOwner: TComponent);
begin
 inherited;
 FCollection:=CreateCollection;
end;

destructor TUnCustomUpdater.Destroy;
begin
 if FCollection <> nil then
  FCollection.Free;
 {if DataSet <> nil then
  DataSet.FUpdater:=nil;}
 inherited;
end;

{-----------------------------TUnDataSet---------------------------------}

constructor TUnDataSet.Create(AOwner: TComponent);
begin
 inherited;
 BookmarkSize:=Sizeof(Integer);
 FStopScroll:=false;
 FIsDescending:=false;
 FOpenFromStreamNow:=false;
 FIsLoaded:=false;
 FOldRecNoForBlobs:=-1;
 RecordPos:=-1;
 FListBlobs:=TList.Create;
 FCursor:=crSQLWait;
 FShowCursor:=false;
 FRecordSet:=nil;
 FIsOpen:=false;
 FBatchMode:=false;
 FOnPostNow:=false;
 //FListCalckFields:=TList.Create;
 //    
 FCompareFieldes[Byte(ftUnknown)]:=CompareError;
 FCompareFieldes[Byte(ftString)]:=CompareString;
 FCompareFieldes[Byte(ftSmallint)]:=CompareSmallInt;
 FCompareFieldes[Byte(ftInteger)]:=CompareInteger;
 FCompareFieldes[Byte(ftWord)]:=CompareWord;
 FCompareFieldes[Byte(ftBoolean)]:=CompareBool;
 FCompareFieldes[Byte(ftFloat)]:=CompareFloat;
 FCompareFieldes[Byte(ftCurrency)]:=CompareFloat;
 FCompareFieldes[Byte(ftBCD)]:=CompareError;
 FCompareFieldes[Byte(ftDate)]:=CompareInteger;
 FCompareFieldes[Byte(ftTime)]:=CompareInteger;
 FCompareFieldes[Byte(ftDateTime)]:=CompareDateTime;
 FCompareFieldes[Byte(ftBytes)]:=CompareError;
 FCompareFieldes[Byte(ftVarBytes)]:=CompareError;
 FCompareFieldes[Byte(ftAutoInc)]:=CompareInteger;
 FCompareFieldes[Byte(ftBlob)]:=CompareError;
 FCompareFieldes[Byte(ftMemo)]:=CompareError;
 FCompareFieldes[Byte(ftGraphic)]:=CompareError;
 FCompareFieldes[Byte(ftFmtMemo)]:=CompareError;
 FCompareFieldes[Byte(ftParadoxOle)]:=CompareError;
 FCompareFieldes[Byte(ftDBaseOle)]:=CompareError;
 FCompareFieldes[Byte(ftTypedBinary)]:=CompareError;
end;
            
destructor TUnDataSet.Destroy;
begin
 inherited;
 FListBlobs.Free;
 //FListCalckFields.Free;
end;

procedure TUnDataSet.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited;
 if Operation = opRemove then
  begin
   if RecordSetDispetcher = AComponent then
     RecordSetDispetcher:=nil
   else if FUpdater = AComponent then
    FUpdater:=nil;
  end;
end;

function TUnDataSet.BookmarkValid(Bookmark: TBookmark): Boolean;  //added by Du
begin
  Result := FRecordSet.FindRecordByID(Integer(Bookmark^)) <> nil;
end;

procedure TUnDataSet.Clear;
begin
 if FRecordSet <> nil then
  FRecordSet.Clear;
 FListBlobs.Clear;
 if FListOffsets <> nil then
  FreeMem(FListOffsets);
 FListOffsets:=nil;
 if FClearBuffer <> nil then
  FreeMem(FClearBuffer);
 FClearBuffer:=nil;
 if FOldBuffer <> nil then
  FreeMem(FOldBuffer);
 FOldBuffer:=nil;
 //FListCalckFields.Clear;
end;

function TUnDataSet.AllocRecordBuffer: PChar;
begin  //    DataSet
 GetMem(Result, FBufferSize);
 InternalInitRecord(Result);
end;

procedure TUnDataSet.FreeRecordBuffer(var Buffer: PChar);
begin   //  
 FreeMem(Buffer, FBufferSize);
 Buffer:=nil;
end;

procedure TUnDataSet.InternalInitRecord(Buffer: PChar);
begin //  
 Move(FClearBuffer^, Buffer^, FBufferSize);
end;

function TUnDataSet.GetRecordSize: Word;
begin //      
 Result:=FRecordSize;
end;

function TUnDataSet.GetRecord(Buffer: PChar; GetMode: TGetMode; DoCheck: Boolean): TGetResult;
var Accept: Boolean;
begin   //  .   !   !
  Result := grOk;
  //Accept := True;
  _CheckPositionCursor(RecordPos);
  case GetMode of
   gmPrior: begin   // 
      if RecordPos <= 0 then
       begin  //  
        Result:=grBof;
        RecordPos:=-1;
       end
      else
       begin
        repeat //  
         RecordPos:=RecordPos-1;
         Accept := _RecordFilter;
         //if Filtered then Accept := _RecordFilter;
        until Accept or (RecordPos =  0);
        if not Accept then
         begin
          Result := grBOF;
          RecordPos := -1;
        end;
       end;
    end;
   gmCurrent: begin  // 
     if (RecordPos < 0) or (RecordPos >= RecordCount) then
      Result:=grError
     else if Filtered then
      if not _RecordFilter then Result := grError;
    end;
   gmNext: begin  // 
      if (RecordPos >= RecordCount - 1) then
        Result:=grEof
      else
       begin
        repeat  //  
         RecordPos:=RecordPos+1;
         Accept := _RecordFilter;
         //if Filtered then Accept := _RecordFilter;
        until Accept or ((RecordPos > RecordCount - 1) and FIsFechAll);
        if not Accept then
         begin
          Result := grEOF;
          RecordPos := RecordCount - 1;
         end;
       end;
    end;
  end;
  if Result = grOk then
   begin //    
    if RecordPos >= FRecordSet.Count then
      FRecordSet.CopyRecordToBuffer(FRecordSet[RecordPos-1], Buffer)
    else
     begin
      if (RecordPos < 0) and (RecordCount > 0) then RecordPos := 0;
      FRecordSet.CopyRecordToBuffer(FRecordSet[RecordPos], Buffer);
     end;
    GetCalcFields(Buffer);
   end
  else
   if (Result = grError) and DoCheck then
    DatabaseError(str_No_Record);
end;

procedure TUnDataSet._CheckPositionCursor(CurPos: Integer);
begin
 if FIsFechAll then exit;
 while (CurPos >= FRecordSet.Count - 1) do
  begin
   if FIsFechAll then
    break;
   _CheckNextFech;
  end;
end;

function TUnDataSet._RecordFilter: Boolean;
var TempBuf: PChar;
    Rec: Pointer;
    OldState : TDataSetState;
begin  // ,   ,     .
{ Result:=true;
 OldState:=State;
 if Filtered then
  try
   SetTempState(dsFilter);
   if _GetActiveRecBuf(TempBuf) then
    begin
     if (RecordPos < FRecordSet.Count) and (RecordPos >= 0 ) then
      begin
       Rec:=FRecordSet[RecordPos];
       if FBatchMode and (PUnRecordData(Rec)^.BatchState = ubs_Delete) then
        begin
         Result:=false;
         Exit;
        end;
       FRecordSet.CopyRecordToBuffer(Rec, TempBuf);
       if Assigned(OnFilterRecord) and (RecordPos > -1) then
        OnFilterRecord(Self, Result);
      end
     else Result:=false;
    end;
  finally
   RestoreState(OldState);
  end;}
 Result:=true;
 OldState:=State;
// if Filtered then // -21.11.2004 atruhin
  try
   SetTempState(dsFilter);
   if _GetActiveRecBuf(TempBuf) then
    begin
     if (RecordPos < FRecordSet.Count) and (RecordPos >= 0 ) then
      begin
       Rec:=FRecordSet[RecordPos];
       if FBatchMode and (PUnRecordData(Rec)^.BatchState = ubs_Delete) then
        begin
         Result:=false;
         Exit;
        end;
       if Filtered then begin // +21.11.2004 atruhin
         FRecordSet.CopyRecordToBuffer(Rec, TempBuf);
         if Assigned(OnFilterRecord) and (RecordPos > -1) then
          OnFilterRecord(Self, Result);
       end;                   // +21.11.2004 atruhin
      end
     else Result:=false;
    end;
  finally
   RestoreState(OldState);
  end;

end;

function TUnDataSet._GetActiveRecBuf(var RecBuf: PChar): Boolean;
begin  //       
  case State of
   dsBrowse: if IsEmpty then
               RecBuf := nil
             else
               RecBuf := ActiveBuffer;
   dsEdit, dsInsert: RecBuf:=ActiveBuffer;
   dsCalcFields: RecBuf:=CalcBuffer;
   dsFilter: RecBuf:=TempBuffer;
   dsOldValue: RecBuf:=FOldBuffer;
   else RecBuf:=nil;
  end;
 Result := RecBuf <> nil;
end;

procedure TUnDataSet._CheckNextFech;
begin //   .     ,    
 FIsFechAll:=true;
end;

function TUnDataSet.GetFieldData(Field: TField; Buffer: Pointer): Boolean;
var RecBuf, Data: PChar;
    SizeBuf: Integer;
begin //       
 if Field.IsBlob then //   
  SizeBuf:=RecordSet.BlobSize
 else
  SizeBuf:=Field.DataSize;
 Result:=false;
 if _GetActiveRecBuf(RecBuf) then //    
  begin
   Data:=_FindFieldData(RecBuf, Field); //   
   if Field.IsBlob then
    begin
     Result:=true;
     if ( Data <> nil ) and ( Buffer <> nil ) then
      Move(Data^, Buffer^, SizeBuf);
    end
   else
    begin
     Result:=PAddData(Data+SizeBuf)^.IsData;
     if Result {and ( Data <> nil ) and ( Buffer <> nil )} then
     begin
      if ( Data <> nil ) and ( Buffer <> nil ) then
       Move(Data^, Buffer^, SizeBuf); // , 
     end;
    end;
  end;
end;

procedure TUnDataSet.SetFieldData(Field: TField; Buffer: Pointer);
var RecBuf, Data: PChar;
    SizeBuf: Integer;
begin  //      
 _CheckWriteMode;  //    
 if Field.IsBlob then //   
  SizeBuf:=RecordSet.BlobSize
 else
  SizeBuf:=Field.DataSize;
 if _GetActiveRecBuf(RecBuf) then
  begin  //    
   Field.Validate(Buffer);
   Data:=_FindFieldData(RecBuf, Field); //   
   if Data <> nil then
    begin
     if (Buffer = nil) then
      begin
       if not Field.IsBlob then
        PAddData(Data+SizeBuf)^.IsData:=false;
      end
     else
      begin
       Move(Buffer^, Data^, SizeBuf);  //  
       if not Field.IsBlob then
        PAddData(Data+SizeBuf)^.IsData := true;
      end;
     if not FIsLoaded and not (State in [dsCalcFields, dsFilter, dsNewValue]) then
      DataEvent(deFieldChange, Longint(Field));  //  ,   
    end;
  end;
end;

function TUnDataSet._FindFieldData(Buffer: Pointer; Field: TField): Pointer;
begin //   
 if Field.Calculated then
  Result:=(PChar(Buffer)+FCalckPosition+Field.Offset)
 else
  Result:=(PChar(Buffer)+FListOffsets[Field.FieldNo]);
end;

procedure TUnDataSet._CheckWriteMode;
begin  //    
 if not (State in dsWriteModes) and not IsLoaded then
  raise TUnDbException.Create(str_Not_Edit_Mode);
end;

procedure TUnDataSet.InternalFirst;
begin
 RecordPos:=-1;
end;

procedure TUnDataSet.InternalLast;
begin
 if not FIsFechAll then
  _FechAll;  //   
 FRealRecordPos:=RecordCount;
end;

procedure TUnDataSet._FechAll;
var OldCursor: TCursor;
begin //  
 OldCursor:=Screen.Cursor;
 try
  if ShowCursor then
   Screen.Cursor:=Cursor;
  FIsRunAllFecher:=true;
  while not FIsFechAll do  //    - 
   _CheckNextFech;
 finally
  FIsRunAllFecher:=false;
  Screen.Cursor:=OldCursor;
 end;
end;

procedure TUnDataSet.SetRecordPosition(AValue: Integer);
begin //      Value
 if Active then
  _CheckPositionCursor(AValue);
  FRealRecordPos := AValue;
 if ( FRealRecordPos > -1 ) and ( FRealRecordPos < FRecordSet.Count ) then
  begin
  PUnRecordData(FRecordSet[FRealRecordPos])^.Position:=FRealRecordPos;
 end;
end;

procedure TUnDataSet.InternalAddRecord(Buffer: Pointer; Append: Boolean);
var Rec: PUnRecordData;
begin  //   
 if Append then
  begin                               //  
   Rec:=FRecordSet.AddNewItem(Buffer, FRecordSet.Count); //   
   if FBatchMode then
    Rec^.BatchState:=ubs_Insert;
   InternalLast;  //   , ..   
  end
 else
  begin                               //    
   Rec:=FRecordSet.AddNewItem(Buffer, RecordPos);
   if FBatchMode then
    Rec^.BatchState:=ubs_Insert;
  end;
end;

procedure TUnDataSet.InternalDelete;
var Index: Integer;
    OldCursor: TCursor;
    Action: TDataAction;
    D: EDatabaseError;
    Accept : boolean;
begin //  
 if RecordPos < 0 then //     
  Index:=0  // , , 
 else if RecordPos > RecordCount - 1 then
  Index:=RecordCount - 1  //    
 else
  Index:=RecordPos; //  - 
 if FBatchMode then  //   -   
  begin
   FRecordSet[Index].BatchState:=ubs_Delete;
   if RecordCount > 0 then  // +21.11.2004 atruhin  
    begin
     repeat //    
      RecordPos:=RecordPos+1;
      Accept := _RecordFilter;
     until Accept or ((RecordPos > RecordCount - 1) and FIsFechAll);
     if not Accept then //     
      repeat
       RecordPos:=RecordPos-1;
       Accept := _RecordFilter;
      until Accept or (RecordPos =  0);
     if not Accept then
      RecordPos := -1;
    end;                          // +21.11.2004 atruhin 
  end
 else
  begin
   if FUpdater <> nil then
    begin
     OldCursor:=Screen.Cursor;
     try
      if ShowCursor then
       Screen.Cursor:=Cursor;
      Action:=daRetry;
      while Action = daRetry do
       try
        FUpdater.DoDelete; //   
        Action:=daAbort;
       except on E: Exception do
        begin
         if Assigned(OnDeleteError) then
          begin
           Action:=daFail;
           D:=EDatabaseError.Create(E.Message);
           OnDeleteError(Self, D, Action);
           if Action = daFail then
            raise;
          end;
        end;
       end;
     finally
      Screen.Cursor:=OldCursor;
     end;
    end;
   FRecordSet.Delete(Index);  // , 
  end;
 RecordPos:=FRealRecordPos; //   :-)
 FIsDeleted:=true; // ,  
 FRecordSet.RecalckPosition;
end;

procedure TUnDataSet.DoBeforeEdit;
begin
 Move(ActiveBuffer^, FOldBuffer^, FBufferSize);
 if not FBatchMode and (FUpdater <> nil) then
  if FUpdater.LockBeforeEdit then
   FUpdater.DoLock;
 inherited;
end;

procedure TUnDataSet.DoAfterEdit;
begin
 if not FBatchMode and (FUpdater <> nil) then
  if FUpdater.RefreshBeforeEdit then
   FUpdater.DoRefresh;
 inherited;
end;

procedure TUnDataSet.InternalPost;
var RecData: Pointer;
    Index: Integer;
    OldCursor: TCursor;
begin // 
 FOnPostNow:=true;
 OldCursor:=Screen.Cursor;
 try
  if ShowCursor then
   Screen.Cursor:=Cursor;
  if (State <> dsInsert) then
   begin  //  
    if RecordPos >= FRecordSet.Count then
     Index:=RecordPos-1  //    
    else
     Index:=RecordPos;  //  - 
    RecData:=FRecordSet[Index];  //    
    if State = dsEdit then  //(?   ?)
     begin
      if FBatchMode then
       begin //   - ,  ,    
        if FRecordSet[Index].BatchState <> ubs_Insert then
         FRecordSet[Index].BatchState:=ubs_Edit;
       end
      else if FUpdater <> nil then
       begin  //    
        FUpdater.DoUpdate;
        _SetBlobsToServer;
       if FUpdater.RefreshAfterPost then  //     ,   
        FUpdater.DoRefresh;
       end;
      FRecordSet.CopyBufferToRecord(RecData, ActiveBuffer); //  
     end;
   end
  else //  
   begin
    if not FBatchMode and (FUpdater <> nil) then
     begin  //    
       FUpdater.DoInsert;
      _SetBlobsToServer;
      if FUpdater.RefreshAfterPost then  //     ,   
       FUpdater.DoRefresh;
     end;
    InternalAddRecord(ActiveBuffer, Eof); //  
   end;
 finally
  Screen.Cursor:=OldCursor;
  FOnPostNow:=false;
 end;
end;

procedure TUnDataSet._SetBlobsToServer;
var I: Integer;
    Fld: TField;
    TempStr: TStream;
    Blob: TUnBlobQuad;
begin //  blob   
 for I:=0 to FListBlobs.Count-1 do
  begin  //    
   Fld:=FListBlobs[I];
   if Fld.GetData(@Blob) then
    if Blob.Blob >= 0 then
     begin  //   .   ,   
      TempStr:=RecordSet.Blob[Blob.Blob];
      if FUpdater <> nil then
       FUpdater.DoSetBlob(Fld as TBlobField, TempStr) //  
      else if Assigned(FOnSetBlob) then
       FOnSetBlob(Self, Fld as TBlobField, TempStr); //   
     end;
  end;
end;

procedure TUnDataSet.GetBookmarkData(Buffer: PChar; Data: Pointer);
begin //   
 PInteger(Data)^:=PUnBookmarkInfo(Buffer + FRecordSize)^.BookmarkData;
end;

procedure TUnDataSet.SetBookmarkData(Buffer: PChar; Data: Pointer);
begin  //   
 if Data <> nil then
  PUnBookmarkInfo(Buffer + FRecordSize)^.BookmarkData := PInteger(Data)^;
end;

function TUnDataSet.GetBookmarkFlag(Buffer: PChar): TBookmarkFlag;
begin //   
 Result:=PUnBookmarkInfo(Buffer + FRecordSize)^.BookmarkFlag;
end;

procedure TUnDataSet.SetBookmarkFlag(Buffer: PChar;
  Value: TBookmarkFlag);
begin  //   
 PUnBookmarkInfo(Buffer + FRecordSize)^.BookmarkFlag:=Value;
end;

procedure TUnDataSet.InternalGotoBookmark(Bookmark: Pointer);
var Rec: PUnRecordData;
    SavePos: Integer;
    Accept: Boolean;
begin //   
  Rec:=FRecordSet.FindRecordByID(Integer(Bookmark^)); //     Id ...
  if (Rec <> nil) then
   begin
    Accept:=True;
    SavePos:=RecordPos;
    try
     RecordPos:=Rec^.Position;  //     
     if Filtered then Accept:=_RecordFilter;  //  ,  ?
    finally
      if not Accept then RecordPos:=SavePos;  //    - 
    end;
   end;
end;

procedure TUnDataSet.InternalSetToRecord(Buffer: PChar);
begin
 InternalGotoBookmark(@PUnBookmarkInfo(Buffer+ FRecordSize)^.BookmarkData);
end;

procedure TUnDataSet.InternalClose;
begin //  
 SetState(dsInactive);
 FIsOpen:=false;  
 if DefaultFields then
  DestroyFields;
 Clear; // 
 if FRecordSet <> nil then
  begin
   FRecordSet.Free;
   FRecordSet:=nil;
  end;
 FRealRecordPos:=-1;
end;

procedure TUnDataSet.InternalOpen;
var OldCursor: TCursor;
    I: Integer;
begin //  
 if FOpenFromStreamNow then
  Exit;
 OldCursor:=Screen.Cursor;
 try
  if ShowCursor then
   Screen.Cursor:=Cursor;
  if FRecordSetDispatcher = nil then
   begin
    FRecordSet:=TUnStreamRecordSet.Create;
    FRecordSet.DataSet:=Self;
    TUnStreamRecordSet(FRecordSet).Stream:=GetTempStream;
    FRecordSet.Capasity:=1;
    TUnStreamRecordSet(FRecordSet).CashMode:=False;
    TUnStreamRecordSet(FRecordSet).BufferSize:=1;
   end
  else
   FRecordSet:=FRecordSetDispatcher.GetRecordSet(Self);  //   
  FIsRunAllFecher:=false;
  RecordPos:=-1;
  FIsFechAll:=false;
  FLoadFieldDefsFromStream:=false;
  InternalInitFieldDefs; //  
  if DefaultFields then
   CreateFields;
  if FLoadFieldDefsFromStream then
   for I:=0 to self.FieldDefs.Count-1 do
    Fields[I].Calculated:=FieldDefs.Items[I].InternalCalcField;
  BindFields(true); //    FieldNo  Offset,    :-)
  _CreateBuffers;
  AlocFetchBuffer;
  FSortBy:='';
  _OpenData;    //  
  FIsDeleted:=false; // ,   
  FIsOpen:=true;
 finally
  Screen.Cursor:=OldCursor;
 end;
end;

procedure TUnDataSet._CreateBuffers;
var I: Integer;
    Fld: TField;
    Offs: Integer;
    BlobCount: Integer;
    Blob: TUnBlobQuad;
begin   //   
 BlobCount:=0;
 try
  Offs:=0;
  if FListOffsets <> nil then
   FreeMem(FListOffsets);
  GetMem(FListOffsets, FieldCount*Sizeof(Integer));
  FCalckFieldSize:=0;
  for I:=0 to FieldCount-1 do
   begin //   
    Fld:=Fields[I];
    if Fld.Calculated then
      FCalckFieldSize:=FCalckFieldSize+Fld.DataSize+SizeOf(TAddData)
    else
     begin
      FListOffsets[Fld.FieldNo]:=Offs;  //      
      if Fld.IsBlob  then
       begin   //   blob,     blob 
        Offs:=Offs+RecordSet.BlobSize;
        FListBlobs.Add(Fld);
        BlobCount:=BlobCount+1;
       end
      else
       Offs:=Offs+Fld.DataSize+SizeOf(TAddData);
     end;
   end;
  FRecordSize:=Offs; //   
  FBufferSize:=FRecordSize+FCalckFieldSize+Sizeof(TUnBookmarkInfo); //  
  FCalckPosition:=FRecordSize+Sizeof(TUnBookmarkInfo);
  GetMem(FClearBuffer, FBufferSize); //   
  GetMem(FOldBuffer, FBufferSize); //      
  FillChar(FClearBuffer^, FBufferSize, 0); //  
  FillChar(FOldBuffer^, FBufferSize, 0); //      -   ...
  Blob.Blob:=-1;
  for I:=0 to FListBlobs.Count-1 do
   begin //     
    Fld:=FListBlobs[I];
    Move(Blob, (FClearBuffer+FListOffsets[Fld.FieldNo])^, Sizeof(TUnBlobQuad));
   end;
 finally
  FBlobPresents:=BlobCount > 0;
 end;
end;

procedure TUnDataSet.ClearCalcFields(Buffer: PChar);
begin
 FillChar((Buffer+FCalckPosition)^, FCalckFieldSize, 0);
end;

procedure TUnDataSet.InternalHandleException;
begin
 Application.HandleException(Self);
end;

procedure TUnDataSet.InternalInitFieldDefs;
begin  //  
 if Assigned(FGetFieldDef) then //  ,   
  begin
   FieldDefs.Clear;
   FGetFieldDef(Self);
  end;
end;

function TUnDataSet.GetRecordCount: Integer;
begin
 _CheckPositionCursor(RecordPos);
  Result:=FRecordSet.Count;
end;

function TUnDataSet.GetRecNo: Integer;
begin //      
 CheckActive;
 _CheckPositionCursor(FRealRecordPos);
 UpdateCursorPos;
 if (FRealRecordPos = -1) and (RecordCount > 0) then Result := 1
 else Result := FRealRecordPos + 1;
end;

procedure TUnDataSet.SetRecNo(Value: Integer);
begin  //     - , ..      
  CheckBrowseMode;
  if (Value < 1) then
    Value := 1
  else if Value > RecordCount then begin
    InternalLast;
    Value := _Min(RecordCount, Value);
  end;
  if (Value <> RecNo) then begin
    DoBeforeScroll;  // 
    RecordPos := Value - 1;
    Resync([]);
    DoAfterScroll;
  end;
end;

procedure TUnDataSet.DoBeforeScroll;
begin
 if not FStopScroll then
  inherited;
end;

procedure TUnDataSet.DoAfterScroll;
begin
 if not FStopScroll then
  inherited;
end;

function TUnDataSet.CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
begin  //    blob 
 if (Mode in [bmWrite, bmReadWrite]) and  not (State in [dsInsert, dsEdit]) then
  if not FIsLoaded then
   raise Exception.Create(str_Not_Edit_Mode); //     ,       
 Result:=TUnMemoryStream.Create(Field as TBlobField, Mode);
end;

function TUnDataSet.GetIsLoaded: boolean;
begin  //    ,         .     
 Result:=FIsLoaded;
end;

procedure TUnDataSet.StartBatchMode;
begin   //   
 FBatchMode:=true;
end;

procedure TUnDataSet.CancelBatchMode;
begin  //    -   
 Refresh;
 FBatchMode:=false;
end;

procedure TUnDataSet.UpdateBatch(IsAll: boolean);
var Bookmark: TBookmark;
    I: Integer;
begin  //    
 try
  DisableControls;
  FBatchMode:=false;  //    
  if IsAll then
   begin
    Bookmark:=GetBookmark; //   
    try
     try
      try
       FStopScroll:=true;
       First;  //   
       I:=0;
       while not Eof do
        begin
         FRealRecordPos:=I;
         SetDataToServer;  //  
         {Next;
         I:=I+1;}

         if I < FRecordSet.Count then
         begin
           if FRecordSet[I]^.BatchState = ubs_None then
           begin
             Next;
             I := I + 1;
           end;
         end
         else Break;

        end;
      finally
       FStopScroll:=false;
      end;
     finally
       if BookmarkValid(Bookmark) then
         GotoBookmark(Bookmark);  //  
     end;
    finally
     FreeBookmark(Bookmark);
    end;
   end
  else
   SetDataToServer;  //      
  inherited;
 finally
  EnableControls;
 end;
 FBatchMode:= not IsAll or Filtered; //   ,    
 if not FBatchMode then
  ClearBlobsStream;  //   .   ?  - !!!!!!!!!
end;

procedure TUnDataSet.ClearBlobsStream;
var Bookmark: TBookmark;
    I: Integer;
    Fld: TField;
    Blob: TUnBlobQuad;
begin
 if FBlobPresents then
  begin
   Bookmark:=GetBookmark;
   try
    try
     DisableControls;
     try
      try
       FStopScroll:=true;
       First;
       while not Eof do
        begin
         for I:=0 to FListBlobs.Count-1 do
          begin
           Fld:=FListBlobs[I];
           if Fld.GetData(@Blob) then
            if Blob.Blob >= 0 then
             begin
              RecordSet.Blob[Blob.Blob].Free;
              Blob.Blob:=-1;
              try
               FIsLoaded:=true;
               Fld.SetData(@Blob);
              finally
               FIsLoaded:=false;
              end;
             end;
          end;
         Next;
        end;
      finally
       FStopScroll:=false;
      end;
     finally
      GotoBookmark(Bookmark);
     end;
    finally
     EnableControls;
    end;
   finally
    FreeBookmark(Bookmark);
   end;
  end;
end;

procedure TUnDataSet.DoBeforeClose;
begin
 inherited;
 if FBatchMode then
  begin  //     ,    
   FBatchMode:=false;
   Filtered:=false;
   ClearBlobsStream;
  end;
end;

//---------------------   ------------------------------------//
                                                                              //
function TUnDataSet.CompareInteger(Buffer1, Buffer2: PChar): Integer;         //
begin                                                                         //
 Result:=PInteger(Buffer1)^-PInteger(Buffer2)^;                               //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareString(Buffer1, Buffer2: PChar): Integer;          //
var Str1: string;                                                             //
    Str2: string;                                                             //
    Len: Integer;                                                             //
    Len1, Len2: Integer;                                                      //
begin                                                                         //
 if FCaseInsensitive then                                                     //
  begin                                                                       //
   Str1:=string(Buffer1);                                                     //
   Str2:=string(Buffer2);                                                     //
  end                                                                         //
 else                                                                         //
  begin                                                                       //
   Str1:=AnsiUpperCase(string(Buffer1));                                      //
   Str2:=AnsiUpperCase(string(Buffer2));                                      //
  end;                                                                        //
 if FPartialKey then                                                          //
  begin                                                                       //
   Len1:=Length(Str1);                                                        //
   Len2:=Length(Str2);                                                        //
   Len:=_Min(Len1, Len2);                                                     //
   if Len > 0 then                                                            //
    Result:=CompareStr(Copy(Str1, 1, Len), Copy(Str2, 1, Len))                //
   else                                                                       //
    Result:=Len1 - Len2;                                                      //
  end                                                                         //
 else                                                                         //
  Result:=CompareStr(Str1, Str2);                                             //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareFloat(Buffer1, Buffer2: PChar): Integer;           //
begin                                                                         //
 if PDouble(Buffer1)^ = PDouble(Buffer2)^ then                                //
  Result:=0                                                                   //
 else if PDouble(Buffer1)^ >= PDouble(Buffer2)^ then                          //
       Result:=1                                                              //
      else                                                                    //
       Result:= -1;                                                           //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareDateTime(Buffer1, Buffer2: PChar): Integer;        //
begin                                                                         //
 Result:=Round(PDateTime(Buffer1)^-PDateTime(Buffer2)^);                      //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareBool(Buffer1, Buffer2: PChar): Integer;            //
begin                                                                         //
 Result:=Word(Buffer1^)-Word(Buffer2^);                                       //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareSmallInt(Buffer1, Buffer2: PChar): Integer;        //
begin                                                                         //
 Result:=PSmallInt(Buffer1)^-PSmallInt(Buffer2)^;                             //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareWord(Buffer1, Buffer2: PChar): Integer;            //
begin                                                                         //
 Result:=PWord(Buffer1)^-PWord(Buffer2)^;                                     //
end;                                                                          //
                                                                              //
function TUnDataSet.CompareError(Buffer1, Buffer2: PChar): Integer;           //
begin                                                                         //
 raise Exception.Create(str_Not_Support);                                     //
end;                                                                          //
//----------------------------------------------------------------------------//

procedure TUnDataSet.SortOnFields(const FieldNames: string;        // FieldNames
                                              ACaseInsensitive,    //   
                                              APartialKey,         //   
                                              ADescending: Boolean //  
                                        );
var OldCurr: TCursor;
    Bmk: TBookmark;
    AFieldList: TList;
    IsCalck: boolean;
begin              //     
 OldCurr:=Screen.Cursor;
 try
  if FShowCursor then
   Screen.Cursor:=FCursor;
   DisableControls;
   _FechAll;
   Bmk:=GetBookmark;
   try
     FIsDescending:=ADescending;
     FCaseInsensitive:=ACaseInsensitive;
     FPartialKey:=APartialKey;
     AFieldList:=TList.Create;
     try
      _CreateListFields(FieldNames, AFieldList, IsCalck);
      FSortBy:=FieldNames;
      _QuickSort(AFieldList, FRecordSet, 0, FRecordSet.Count-1, _CompareFields);
      FRecordSet.Pack;
     finally
      AFieldList.Free;
     end;
    Resync([]);
    GotoBookmark(Bmk);
   finally
    FreeBookmark(Bmk);
   end;
  finally
   Screen.Cursor:=OldCurr;
   EnableControls;
  end;
 DataEvent(deDataSetChange, 0);
end;

procedure TUnDataSet._CreateListFields(const AIndexName: string; ListFields: TList; var IsCalck: boolean);
var I: Integer;
    LenName: Integer;
    Fld: TField;
    TempName: string;
begin  //     
 LenName:=Length(AIndexName);
 TempName:='';
 for I:=1 to LenName do
  begin
   if AIndexName[I] = ';' then
    begin
     Fld:=FieldByName(TempName);
     if Fld.IsBlob then
      raise Exception.Create(str_Not_Blod_To_Index);
     ListFields.Add(Fld);
     TempName:='';
    end
   else
    TempName:=TempName+AIndexName[I];
  end;
 if TempName <> '' then
  begin
   Fld:=FieldByName(TempName);
   if Fld.IsBlob then
    raise Exception.Create(str_Not_Blod_To_Index);
   ListFields.Add(Fld);
  end;
end;

procedure TUnDataSet._QuickSort(AListFields: TList; SortList: TUnCustomRecordSet; L, R: Integer; SCompare: TUnFieldCompare);
var
  I, J: Integer;
  P: Pointer;
  Fld: TField;
  K: Integer;
  FOldPosition: Integer;
begin  // , 
 if RecordCount > 0 then
  begin
   FIsRecalck:=false;
   for K:=0 to AListFields.Count-1 do //   ,      
    begin
     Fld:=AListFields[K];
     if Fld.Calculated then
      begin
       FIsRecalck:=true;
       break;
      end;
    end;
   FOldPosition:=RecordPos;
   try
    repeat
     I := L;
     J := R;
     K:=(L + R) shr 1;
     P := SortList[K];
     repeat
       while SCompare(AListFields, SortList[I], P, I, K) < 0 do
         Inc(I);
       while SCompare(AListFields, SortList[J], P, J, K) > 0 do
         Dec(J);
       if I <= J then
        begin
         FRecordSet.Exchange(I, J);
         Inc(I);
         Dec(J);
        end;
     until I > J;
     if L < J then
       _QuickSort(AListFields, SortList, L, J, SCompare);
     L := I;
    until I >= R;
   finally
    RecordPos:=FOldPosition;
   end;
  end;
end;

function TUnDataSet._CompareFields(ListFields: TList; P1, P2: Pointer; I1, I2: Integer): Integer;
var Rec1: PUnRecordData;
    Rec2: PUnRecordData;
    Fld: TField;
    Buf1: PChar;
    Buf2: PChar;
    I: Integer;
    FSortBuffer1: PChar;
    FSortBuffer2: PChar;
begin  //  
 FSortBuffer1:=AllocRecordBuffer;
 try
  FSortBuffer2:=AllocRecordBuffer;
  try
   Result:=0;
   Rec1:=P1;
   Rec2:=P2;
   if FIsRecalck then
    begin
     RecordPos:=I1;
     FRecordSet.CopyRecordToBuffer(Rec1, FSortBuffer1);
     RecordPos:=I2;
     FRecordSet.CopyRecordToBuffer(Rec2, FSortBuffer2);
    end
   else
    begin
     FRecordSet.CopyRecordToBuffer(Rec1, FSortBuffer1);
     FRecordSet.CopyRecordToBuffer(Rec2, FSortBuffer2);
    end;
   for I:=0 to ListFields.Count-1 do
    begin
     Fld:=ListFields[I];
     Buf1:=_FindFieldData(FSortBuffer1, Fld);
     Buf2:=_FindFieldData(FSortBuffer2, Fld);
     Result:=FCompareFieldes[Byte(Fld.DataType)](Buf1, Buf2);
     if FIsDescending then
      Result:=-Result;
     if Result <> 0 then
      break;
    end;
  finally
   FreeRecordBuffer(FSortBuffer2);
  end
 finally
  FreeRecordBuffer(FSortBuffer1);
 end;
end;

function TUnDataSet.Locate(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
var FieldLst: TList;
    AIsCalck: boolean;
    ACaseInsensitive: boolean;
    APartialKey: boolean;
var OldCurr: TCursor;
begin  //  
 DisableControls;
 try
  OldCurr:=Screen.Cursor;
  try
   if FShowCursor then
    Screen.Cursor:=FCursor;
   FieldLst:=TList.Create;
   try
    _CreateListFields(KeyFields, FieldLst, AIsCalck);
    ACaseInsensitive:=loCaseInsensitive in Options;
    APartialKey:=loPartialKey in Options;
    if FSortBy = KeyFields then
     begin
      if (ACaseInsensitive <> FCaseInsensitive) or (FPartialKey <> APartialKey) then
       SortOnFields(KeyFields, ACaseInsensitive, APartialKey, self.FIsDescending);
      Result:=_QuickLocate(FieldLst, KeyValues, ACaseInsensitive, APartialKey);
     end
    else
     Result:=_SlowlyLocate(FieldLst, KeyValues, ACaseInsensitive, APartialKey);
   finally
    FieldLst.Free;
   end;
  finally
   Screen.Cursor:=OldCurr;
  end;
 finally
  EnableControls;
 end;
end;

function TUnDataSet._QuickLocate(FieldLst: TList; const KeyValues: Variant; ACaseInsensitive, APartialKey: boolean): boolean;
var OldCaseInsensitive: boolean;
    OldPartialKey: boolean;
    Bmk: TBookmark;
    isUp: boolean;
    MaxDown, MaxUp, NextStep, LastComp, PozNow: Integer;
    OldAutoCalck: boolean;
begin // ,      -   
 OldCaseInsensitive:=FCaseInsensitive;
 OldPartialKey:=FPartialKey;
 DisableControls;
 try
  Bmk:=GetBookmark;
  DoBeforeScroll;
  try
   OldAutoCalck:=AutoCalcFields;
   FCaseInsensitive:=ACaseInsensitive;
   FPartialKey:=APartialKey;
   Result:=_CompareRecord(FieldLst, KeyValues) = 0;
   if not Result then
    begin
     try
      FStopScroll:=true;
      First;
      PozNow:=0;
      MaxUp:=0;
      MaxDown:=RecordCount;
      NextStep:=Trunc((MaxDown - MaxUp)/2);
      isUp:=false;
      while true do
       begin
        if isUp then
         begin
          if MaxUp > (PozNow - NextStep) then
           break;
          NextStep:=-NextStep;
         end
        else
         begin
          if MaxDown < (PozNow + NextStep) then
           break;
         end;
        if MaxDown - MaxUp < 5 then
         begin
          NextStep:=PozNow-MaxUp;
          try
           AutoCalcFields:=false;
           MoveBy(-NextStep);
           PozNow:=PozNow-NextStep;
          finally
           AutoCalcFields:=OldAutoCalck;
          end;
          while PozNow <= MaxDown do
           begin
            LastComp:=_CompareRecord(FieldLst, KeyValues);
            Result:=LastComp = 0;
            if Result then
             break;
            Next;
            PozNow:=PozNow+1;
           end;
          break;
         end;
        PozNow:=PozNow+NextStep;
        try
         AutoCalcFields:=false;
         MoveBy(NextStep);
        finally
         AutoCalcFields:=OldAutoCalck;
        end;
        LastComp:=_CompareRecord(FieldLst, KeyValues);
        Result:=LastComp = 0;
        if Result then
         break;
        isUp:=(LastComp < 0);
        if not FIsDescending then
         isUp:=not isUp;
        if isUp then
         MaxDown:=PozNow
        else
         MaxUp:=PozNow;
        NextStep:=Trunc((MaxDown - MaxUp)/2);
       end;
      if not Result then
       GotoBookmark(Bmk);
     finally
      FStopScroll:=false;
     end;
     //Resync([]);
    end;
   finally
    FreeBookmark(Bmk);
   end;
   if Result then
    DoAfterScroll
 finally
  FCaseInsensitive:=OldCaseInsensitive;
  FPartialKey:=OldPartialKey;
  EnableControls;
 end;
end;

function TUnDataSet._SlowlyLocate(FieldLst: TList; const KeyValues: Variant; ACaseInsensitive, APartialKey: boolean): boolean;
var OldCaseInsensitive: boolean;
    OldPartialKey: boolean;
    Bmk: TBookmark;
begin  //    
 OldCaseInsensitive:=FCaseInsensitive;
 OldPartialKey:=FPartialKey;
 DisableControls;
 try
  FCaseInsensitive:=ACaseInsensitive;
  FPartialKey:=APartialKey;
  Bmk:=GetBookmark;
  try
   Result:=_CompareRecord(FieldLst, KeyValues) = 0;
   if not Result then
    begin
     DoBeforeScroll;
     try
      FStopScroll:=true;
      First;
      while not Eof do
       begin
        Result:=_CompareRecord(FieldLst, KeyValues) = 0;
        if Result then
         begin
          DataEvent(deDataSetScroll, 0);
          break;
         end;
        Next;
       end;
      if not Result then
       GotoBookmark(Bmk)
     finally
      FStopScroll:=false;
     end;
     //Resync([]);
    end;
   finally
    FreeBookmark(Bmk);
   end;
  if Result then
   DoAfterScroll
 finally
  FCaseInsensitive:=OldCaseInsensitive;
  FPartialKey:=OldPartialKey;
  EnableControls;
 end;
end;

function TUnDataSet._CompareRecord(ListFields: TList; const KeyValues: Variant): Integer;
var  I: Integer;

   function CompareField(Field: TField; Value: Variant): Integer;
   var S: string;
   begin
     if Field.DataType = ftString then
      begin
       S := Field.AsString;
       if FPartialKey then
         System.Delete(S, Length(Value) + 1, MaxInt);
       if FCaseInsensitive then
         Result := AnsiCompareText(S, Value)
       else
         Result := AnsiCompareStr(S, Value);
      end
     else if Field.IsNull then
      Result := -1
     else Result := (Field.Value - Value);
  end;

begin  //      
 Result:=0;
 if ListFields.Count = 1 then
  Result:=CompareField(TField(ListFields[0]), KeyValues)
 else
  for I := 0 to ListFields.Count - 1 do
   begin
    Result:=CompareField(TField(ListFields[I]), KeyValues[I]);
    if Result <> 0 then
     break;
   end;
end;

procedure TUnDataSet._InternalAddFechData(Buffer: PChar);
begin //        
 FRecordSet.AddNewItem(Buffer, FRecordSet.Count);
end;

procedure TUnDataSet._OpenData;
begin //  
 FRecordSet.Open;
end;

function TUnDataSet.IsCursorOpen: Boolean;
begin  // -  ?
 Result:=FIsOpen;
end;

function TUnDataSet.CompareBookmarks(Bookmark1, Bookmark2: TBookmark): Integer;
begin
 if (Bookmark1 <> nil) and (Bookmark2 <> nil) then
  Result:=PInteger(Bookmark1)^ - PInteger(Bookmark2)^
 else
  Result:=0;
end;

procedure TUnDataSet.OpenFromStream(Str: TStream);
var OldCursor: TCursor;
    FieldDefsPosition: Integer;
begin //    Stream
 if Active then
  raise Exception.Create(str_Alredy_Opened);
 OldCursor:=Screen.Cursor;
 try
  if ShowCursor then
   Screen.Cursor:=Cursor;
  if FRecordSetDispatcher = nil then
   raise TUnDbException.Create(str_Not_Set_RecorsSetDisp);
  FRecordSet:=FRecordSetDispatcher.GetRecordSet(Self);  //   
  DoBeforeOpen;
  try
   FIsRunAllFecher:=false;
   RecordPos:=-1;
   FIsFechAll:=false;
   FieldDefs.Clear;
   Str.Position:=Str.Size - sizeof(Integer);
   Str.Read(FieldDefsPosition, sizeof(Integer)); //    Fields
   Str.Position:=FieldDefsPosition;
   LoadFieldDefsFromStream(FieldDefs, Str);  //  
   FRecordSet.OpenFromStream(Str);
   if (FieldCount = 0) then
    CreateFields;
   BindFields(true); //    FieldNo  Offset,    :-)
   _CreateBuffers;
   AlocFetchBuffer;
   FSortBy:='';
   FIsDeleted:=false; // ,   
   FIsOpen:=true;
   FOpenFromStreamNow:=true;
   try
    OpenCursor(false);    //  
   finally
    FOpenFromStreamNow:=false;
   end;
   SetState(dsBrowse);
  except
   SetState(dsInactive);
   CloseCursor;
   raise;
  end;
  DoAfterOpen;
  DoAfterScroll;
  FIsFechAll:=true; //       ?
 finally
  Screen.Cursor:=OldCursor;
 end;
end;

procedure TUnDataSet.SaveToStream(Str: TStream);
var TempStr: TStream;
    FieldDefPosition: Integer;
begin
 Str.Size:=0;
 Pack;
 TempStr:=FRecordSet.GetStream(false);
 TempStr.Position:=0;
 Str.CopyFrom(TempStr, TempStr.Size);
 FieldDefPosition:=Str.Position;
 SaveFieldDefsToStream(FieldDefs, Str);
 SaveInt(RecordCount, Str);
 SaveInt(FieldDefPosition, Str);     
end;

procedure TUnDataSet.Pack;
begin
 if FIsDeleted then
  FRecordSet.Pack;
end;

procedure TUnDataSet.Empty;
begin
 FRecordSet.Clear;
 FRecordSet.ClearData;
 FRealRecordPos:=0;
end;

procedure TUnDataSet.AddData(const Str: TStream);
begin
 FIsFechAll:=FRecordSet.AddData(Str) = 0;
end;

procedure TUnDataSet.SaveData(const Str: TStream);
begin
 Pack;
 FRecordSet.SaveData(Str);
end;

procedure TUnDataSet.AlocFetchBuffer;
begin
end;

procedure TUnDataSet.FetchAll;
var Bmk: TBookmark;
begin
 if not IsFechAll and (RecordCount > 0) then
  begin
   Bmk:=GetBookmark;
   try
    Last;
    GotoBookmark(Bmk);
   finally
    FreeBookmark(Bmk);
   end;
  end;
end;

{-------------------------------TUnMemoryStream--------------------------------}

constructor TUnMemoryStream.Create(Field: TBlobField; Mode: TBlobStreamMode);
var Blob: TUnBlobQuad;
    NewBl: boolean;
    TempStr: TStream;
begin
 FModified:=false;
 FPosition:=0;
 FMode:=Mode;
 FField:=Field;
 FDataSet:=Field.DataSet as TUnDataSet;
 NewBl:=false;
 if Field.GetData(@Blob) then
  begin  //     
   if Blob.Blob >= 0 then
    FBlob:=Blob.Blob //  
   else
    begin  //   - 
     NewBl:=true;
     FBlob:=FDataSet.RecordSet.GetBlob;
    end;
  end;
 if NewBl then  //   -   
  begin
   Blob.Blob:=FBlob;
   try
    FDataSet.FIsLoaded:=true; //   ,      
    Field.SetData(@Blob);
    TempStr:=FDataSet.RecordSet.Blob[FBlob];
    if FDataSet.FUpdater <> nil then //   -   
     FDataSet.FUpdater.DoGetBlob(FField, TempStr)
    else if Assigned(FDataSet.FOnGetBlob) then //   -   
     FDataSet.FOnGetBlob(FDataSet, FField, TempStr);
    FModified:=false;
    TempStr.Position:=0;
   finally
    FDataSet.FIsLoaded:=false;
   end;
  end
 else if FDataSet.FUpdater.RefreshBeforeEdit then
       if not FDataSet.FOnPostNow and (FDataSet.State = dsEdit) and (Mode in [bmRead, bmReadWrite]) and (FDataSet.FUpdater <> nil) then
        if not TUDBTempFile(FDataSet.RecordSet.Blob[FBlob]).Modified then
         begin
          TempStr:=FDataSet.RecordSet.Blob[FBlob];
          TempStr.Size:=0;
          FDataSet.FUpdater.DoGetBlob(FField, TempStr)
         end;
end;

destructor TUnMemoryStream.Destroy; 
begin
 if FModified then  //  !!!     ,    .
                    //     ,  
                    //    .
  FDataSet.RecordSet.Blob[FBlob].Size:=FPosition;
 inherited;
end;

function TUnMemoryStream.Read(var Buffer; Count: Integer): Longint;
begin //   
 FDataSet.RecordSet.Blob[FBlob].Position:=FPosition;
 Result:=0;
 if FMode <> bmWrite then
  Result:=FDataSet.RecordSet.Blob[FBlob].Read(Buffer, Count);
 FPosition:=FDataSet.RecordSet.Blob[FBlob].Position;
end;

function TUnMemoryStream.Seek(Offset: Integer; Origin: Word): Longint;
begin  //   
 case Origin of
  soFromBeginning: FPosition := Offset;
  soFromCurrent: Inc(FPosition, Offset);
  soFromEnd: FPosition:=FDataSet.RecordSet.Blob[FBlob].Size + Offset;
 end;
 Result:=FPosition;
end;

procedure TUnMemoryStream.Truncate;
begin //   
 SetSize(0);
end;

function TUnMemoryStream.Write(const Buffer; Count: Integer): Longint;
begin  // 
 FDataSet.RecordSet.Blob[FBlob].Position:=FPosition;
 if (FMode <> bmRead) and ((FDataSet.State in [dsEdit, dsInsert]) or FDataSet.FIsLoaded) then
   Result:=FDataSet.RecordSet.Blob[FBlob].Write(Buffer, Count)
 else
  raise Exception.Create(str_Not_Write_Mode);
 FPosition:=FDataSet.RecordSet.Blob[FBlob].Position;
 FField.Modified:=true;
 FModified:=true;
end;

procedure TUnMemoryStream.SetSize(NewSize: Longint);
begin
 FDataSet.RecordSet.Blob[FBlob].Size:=Size;
end;

{--------------------------------TUnMemoryTable--------------------------------}

procedure TUnMemoryTable.InternalInitFieldDefs;
var I: Integer;
    Field: TField;
begin
 for I:=0 to self.FieldCount-1 do
  begin
   Field:=Fields[I];
   TFieldDef.Create(FieldDefs, Field.FieldName,
                  Field.DataType, Field.Size, Field.Required, I+1);
  end;
end;

end.
