
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnFetchTable;

interface

  uses SysUtils, Classes, Db, dbtables, extctrls, UnCustomDataSet, UnRecordSet,
       UnSQLUpdater, UnFileServices;

  Type

   TUnFetchDataSet = class;

   //  ,      
   TUnCustomFetcher = class(TUnSQLUpdater)
    private
     FDataSet: TUnFetchDataSet; // ,      
     FFieldDefData: TStream;
    protected
     FIsFieldsChecked: boolean; //   ,       
     FInitializeFromFieldDefData: boolean;
     procedure SetDataSet(Value: TUnDataSet); override;
     function GetDataSet: TUnDataSet; override;
     function GetActive: boolean; virtual; abstract;
     procedure SetActive(const Value: boolean); virtual; abstract;
     function GetBof: boolean; virtual; abstract;
     function GetEof: boolean; virtual; abstract;
     function GetSQL: TStrings; virtual; abstract;
     procedure SetSQL(const Value: TStrings); virtual; abstract;
     procedure GetNextData(AData: PChar); virtual; abstract;
     procedure InitializeFields(ADataSet: TDataSet); virtual; abstract;
     function GetFieldCount: Word; virtual; abstract;
     function FindField(const AFieldName: string): boolean;
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure Open; virtual; abstract;
     procedure Close; virtual; abstract;
     property Active: boolean read GetActive write SetActive;
     property Eof: boolean read GetEof;
     property Bof: boolean read GetBof;
     property FieldCount: Word read GetFieldCount;
     property IsFieldsChecked: boolean read FIsFieldsChecked;
     property FetchDataSet: TUnFetchDataSet read FDataSet;
     property FieldDefData: TStream read FFieldDefData;
    published
     property SQL: TStrings read GetSQL write SetSQL; //    
     property InitializeFromFieldDefData: boolean read FInitializeFromFieldDefData write FInitializeFromFieldDefData;
   end;

   //   
   TUnDataLink = class(TDataLink)
    private
     FDataSet: TUnDataSet;  // ,    
     FTimer: TTimer;        // ,    ,     
     FHardRefresh: boolean;
     procedure TimerTick(Sender: TObject);  //   
     procedure DoRefresh;                   // 
    protected
     procedure DataSetChanged; override;    //      
     procedure UpdateData; override;        //     
     procedure ActiveChanged; override;
    public
     constructor Create;
     destructor Destroy; override;
     property HardRefresh: boolean read FHardRefresh write FHardRefresh;
   end;

   TUnFetchDataSet = class(TUnDataSet)
    private
     FFetcher: TUnCustomFetcher; //  -  
     FFechBuffer: PChar;         //    
     FMasterDataSource: TDataSource;   //  DataSource  -  ,     FDataLink
     FDataLink: TUnDataLink;           //   
     FParams: TParams;
     FHardRefresh: boolean;
     function GetFieldOffset(Index: Integer): Integer;
     procedure SetMasterDataSource(Value: TDataSource);
     function GetFetcher: TUnCustomFetcher;
     procedure SetHardRefresh(Value: boolean);
    protected
     procedure _CheckNextFech; override;
     procedure SetDataToServer; override;
     procedure InternalOpen; override;
     procedure InternalClose; override;
     procedure DataEvent(Event: TDataEvent; Info: Integer); override;
     procedure InternalRefresh; override;
     procedure SetFetcher(Value: TUnCustomFetcher); virtual;
     procedure SetParamsList(Value: TParams);
     procedure AlocFetchBuffer; override;
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure FetchAll;
     property FieldOffset[Index: Integer]: Integer read GetFieldOffset; //    
    published
     property MasterDataSource: TDataSource read FMasterDataSource write SetMasterDataSource;
     property Fetcher: TUnCustomFetcher read GetFetcher write SetFetcher;
     property Params: TParams read FParams write SetParamsList;
     property HardRefresh: boolean read FHardRefresh write SetHardRefresh;
     property RecordSetDispetcher;
     property Filtered;
     property AutoCalcFields;
     property AfterCancel;
     property AfterClose;
     property AfterDelete;
     property AfterEdit;
     property AfterInsert;
     property AfterOpen;
     property AfterPost;
     property AfterScroll;
     property BeforeCancel;
     property BeforeClose;
     property BeforeDelete;
     property BeforeEdit;
     property BeforeInsert;
     property BeforeOpen;
     property BeforePost;
     property BeforeScroll;
     property OnCalcFields;
     property OnDeleteError;
     property OnEditError;
     property OnNewRecord;
     property OnPostError;
     property OnFilterRecord;
     property Active;
   end;

   var
 str_Not_Assign_Fecher : string = 'Assign fecher before open !';
 str_Not_Fetch_DataSet : string = 'Assign possible only TUnFetchDataSet !';

implementation

{---------------------------------TUnCustomFecher------------------------------}

constructor TUnCustomFetcher.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIsFieldsChecked:=false;  //       
  FDataSet:=nil;
  FFieldDefData:=GetTempStream;
  FInitializeFromFieldDefData:=false;
end;

destructor TUnCustomFetcher.Destroy;
begin
 if (csDesigning in ComponentState) and (FDataSet <> nil) then
 FDataSet.Fetcher:=nil;  //   
 FFieldDefData.Free;
 inherited;
end;

function TUnCustomFetcher.FindField(const AFieldName: string): boolean;
var I: Integer;
begin //   true,    
 Result:=false;
 for I:=0 to FDataSet.FieldCount-1 do
  begin
   if CompareText(FDataSet.Fields[I].FieldName, AFieldName) = 0 then
    begin
     Result:=true;
     break;
    end;
  end;
end;

procedure TUnCustomFetcher.SetDataSet(Value: TUnDataSet);
begin
 if not (Value is TUnFetchDataSet) then
  raise TUnDbException.Create(str_Not_Fetch_DataSet);
 FDataSet:=TUnFetchDataSet(Value);
end;

function TUnCustomFetcher.GetDataSet: TUnDataSet;
begin
 Result:=FDataSet;
end;

{--------------------------------TUnFechDataSet--------------------------------}

constructor TUnFetchDataSet.Create(AOwner: TComponent);
begin
 inherited;
 FParams:=TParams.Create;
 FFetcher:=nil;
 FMasterDataSource:=nil;
end;

destructor TUnFetchDataSet.Destroy;
begin
 inherited;
 FParams.Free;
 if FDataLink <> nil then
   FDataLink.Free;
end;

procedure TUnFetchDataSet.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited;
 if AComponent <> nil then
  if Operation = opRemove then
   begin
    if Fetcher = AComponent then
     Fetcher:=nil;
    if MasterDataSource = AComponent then
     MasterDataSource:=nil;
   end;
end;

procedure TUnFetchDataSet.FetchAll;
begin   //   
 _FechAll;
end;

procedure TUnFetchDataSet.SetMasterDataSource(Value: TDataSource);
begin
 if FMasterDataSource <> Value then //   DataSource
  begin
   if FDataLink <> nil then  //     DataLink
    begin
     if Value = nil then //  DataSource   
      begin
       FDataLink.Free;
       FDataLink:=nil;
       FMasterDataSource:=nil;
      end;
    end
   else // , DataLink  
    if Value.DataSet <> Self then  //     !
     begin
      if FDataLink = nil then
       begin //  DataLink   
        FDataLink:=TUnDataLink.Create;
        FDataLink.FDataSet:=Self;
        FDataLink.HardRefresh:=FHardRefresh;
       end;
      Close;
      FDataLink.DataSource:=Value;
      FMasterDataSource:=Value;
     end;
  end;
end;

procedure TUnFetchDataSet.SetHardRefresh(Value: boolean);
begin
 if FDataLink <> nil then
  FDataLink.HardRefresh:=Value;
 FHardRefresh:=Value;
end;

procedure TUnFetchDataSet.InternalRefresh;
var OldRecNo: Integer;
begin
 inherited;
 try   //      
  DisableControls;
  OldRecNo:=RecNo;
  Close;
  Open;
  RecNo:=OldRecNo;
 finally
  EnableControls;
 end;
end;

procedure TUnFetchDataSet._CheckNextFech;
begin //       
 if FFetcher.Eof then
  begin
   FIsFechAll:=true;  //    , ..   
   FFetcher.Close;
  end
 else
  begin //    
   FFetcher.GetNextData(FFechBuffer);
   _InternalAddFechData(FFechBuffer);
  end;
end;

procedure TUnFetchDataSet.InternalOpen;
begin
 if FFetcher = nil then //    
  raise TUnDbException.Create(str_Not_Assign_Fecher);
 FIsFechAll:=false;
 if not FFetcher.Active then
  FFetcher.Active:=true; //  
 inherited;
end;

procedure TUnFetchDataSet.InternalClose;
begin
 if not (csDestroying in ComponentState) then
  if FFetcher <> nil then  // 
   if FFetcher.Active then //   ,  
    FFetcher.Active:=false;
 if FFechBuffer <> nil then  //  
  FreeRecordBuffer(FFechBuffer);
 FFechBuffer:=nil;
 inherited;
end;

procedure TUnFetchDataSet.AlocFetchBuffer;
begin
 if FFechBuffer <> nil then
  FreeRecordBuffer(FFechBuffer);
 if FFechBuffer = nil then  //    
  begin
   FFechBuffer:=AllocRecordBuffer;
  end;
end;

procedure TUnFetchDataSet.DataEvent(Event: TDataEvent; Info: Integer);
begin
 if (Event in [deFieldListChange, deFieldChange]) and (FFetcher <> nil) and
    not (csDestroying in ComponentState)  then
  FFetcher.FIsFieldsChecked:=false;  //          
 inherited;
end;

function TUnFetchDataSet.GetFieldOffset(Index: Integer): Integer;
begin  //    
 Result:=FListOffsets[Index];
end;

procedure TUnFetchDataSet.SetDataToServer;
var Rec:  PUnRecordData;
begin //     -  
 Rec:=FRecordSet[RecordPos];
 case Rec.BatchState of
  ubs_Insert: begin  //  
    if FUpdater <> nil then
     begin
      FUpdater.DoInsert;
      _SetBlobsToServer;
     end;
    Rec^.BatchState:=ubs_None;
   end;
  ubs_Edit: begin   //  
    if FUpdater <> nil then
     begin
      FUpdater.DoUpdate;
      _SetBlobsToServer;
     end;
    Rec^.BatchState:=ubs_None;
   end;
  ubs_Delete: begin  // 
     Delete;
   end;
 end;
end;

procedure TUnFetchDataSet.SetParamsList(Value: TParams);
begin
  CheckInactive;
  FParams.Assign(Value);
end;

function TUnFetchDataSet.GetFetcher: TUnCustomFetcher;
begin
 Result:=FFetcher;
end;

procedure TUnFetchDataSet.SetFetcher(Value: TUnCustomFetcher);
begin
 if Value <> FFetcher then  //   .   ,    
  begin
   if Value = nil then  //    
    begin
     Close;
     FFetcher:=nil;
     FUpdater:=nil;
     OnGetFieldDef:=nil;
    end
   else
    begin //   
     if Value <> nil then //  
      if Value.FDataSet <> nil then  //   
       Value.FDataSet.FFetcher:=nil; //     
     FFetcher:=Value; //   
     FFetcher.FDataSet:=Self;  //     - 
     FUpdater:=Value;
     OnGetFieldDef:=FFetcher.InitializeFields;  //    
    end;
  end;
end;

{----------------------------------TUnDataLink---------------------------------}

constructor TUnDataLink.Create;
begin
 inherited;
 FTimer:=TTimer.Create(nil);
 FTimer.Enabled:=false;
 FTimer.Interval:=350;  //   
 FTimer.OnTimer:=TimerTick;
 FHardRefresh:=false;
end;

destructor TUnDataLink.Destroy;
begin
 FTimer.Free;
 inherited;
end;

procedure TUnDataLink.TimerTick(Sender: TObject);
begin
 try  //  
  FTimer.Enabled:=false;
  FDataSet.DisableControls;
  FDataSet.Close;  //  
  if DataSource.DataSet.Active then
   FDataSet.Open;
 finally
  FDataSet.EnableControls;
 end;
end;

procedure TUnDataLink.DoRefresh;
begin  //  , ,        
 if FDataSet.Active then
  begin
   if FHardRefresh then
    TimerTick(FTimer)
   else
    begin
     FTimer.Enabled:=false;
     FTimer.Enabled:=true;
    end;
  end
 else
  begin
   FTimer.Enabled:=false;
   FDataSet.Close;
  end;
end;

procedure TUnDataLink.DataSetChanged;
begin  // ,   
 inherited;
 DoRefresh;
end;

procedure TUnDataLink.UpdateData;
begin //  Post
 inherited;
 DoRefresh;
end;

procedure TUnDataLink.ActiveChanged;
begin
 inherited;
 DoRefresh;
end;

end.
