
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnFibClasses;

interface

 uses SysUtils, Classes, Db, UnFetchTable, pFIBQuery, pFIBProps, FIBDatabase,
      ibase, fib, FIBQuery, UnConstants, UnCustomDataSet, UnDbFunctions,
      UnRecordSet;

 Type

  TFibFetchFunc = procedure(Fld: TField; IfField: PXSQLVAR; Buffer: PChar);

  TUnCustomFibFetcher = class(TUnCustomFetcher)
   private
    FQuery: TFIBQuery;
    FieldFechIndex: POffsetsArray;
    function GetOptions: TpFIBQueryOptions;
    procedure SetOptions(Value: TpFIBQueryOptions);
   protected
    function GetReadDatabase: TFIBDatabase; virtual;
    procedure SetReadDatabase(Value: TFIBDatabase); virtual;
    function GetWriteDatabase: TFIBDatabase; virtual; abstract;
    procedure SetWriteDatabase(Value: TFIBDatabase); virtual; abstract;
    function GetReadTransaction: TFIBTransaction; virtual;
    procedure SetReadTransaction(Value: TFIBTransaction); virtual;
    function GetWriteTransaction: TFIBTransaction; virtual; abstract;
    procedure SetWriteTransaction(Value: TFIBTransaction); virtual; abstract;
    function GetActive: boolean; override;
    procedure SetActive(const Value: boolean); override;
    function GetBof: boolean; override;
    function GetEof: boolean; override;
    function GetParamCheck: boolean;
    procedure SetParamCheck(const Value: boolean); 
    function GetSQL: TStrings; override;
    procedure SetSQL(const Value: TStrings); override;
    procedure GetNextData(AData: PChar); override;
    procedure InitializeFields(ADataSet: TDataSet); override;
    function GetFieldCount: Word; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Open; override;
    procedure Close; override;
   published
    property ReadDatabase: TFIBDatabase read  GetReadDatabase write SetReadDatabase;
    property WriteDatabase: TFIBDatabase read  GetWriteDatabase write SetWriteDatabase;
    property Options: TpFIBQueryOptions read GetOptions write SetOptions;
    property ReadTransaction: TFIBTransaction read GetReadTransaction write SetReadTransaction;
    property WriteTransaction: TFIBTransaction read GetWriteTransaction write SetWriteTransaction;
    property ParamCheck: boolean read GetParamCheck write SetParamCheck;
  end;

  var
    AFechFuncList: Array[447..583] of  TFibFetchFunc;  //  ,    
    II: Integer;

  const
     IBBuffDateDelta=678576;

implementation

 uses UnMessages;

 Type

    HackDataSet = class(TUnDataSet);

{----------------------------------TUnFibFetcher--------------------------------}

constructor TUnCustomFibFetcher.Create(AOwner: TComponent);
begin
 inherited;
 FQuery:=TFIBQuery.Create(Self);
 FQuery.Name:=Name+'_FIBFecher';
 FQuery.GoToFirstRecordOnExecute:=true;
 FQuery.ParamCheck:=true;
 FQuery.Options:=[];
 FQuery.GoToFirstRecordOnExecute:=true;
end;

destructor TUnCustomFibFetcher.Destroy;
begin
 Close;
 inherited;
end;

procedure TUnCustomFibFetcher.Notification(AComponent: TComponent; Operation: TOperation); 
begin
 inherited;
 if AComponent <> nil then
  if Operation = opRemove then
   begin
    if AComponent = ReadDatabase  then
     begin
      if FQuery.Open then
       FQuery.Close;
      ReadDatabase:=nil;
     end;
    if AComponent = WriteDatabase then
     WriteDatabase:=nil;
    if AComponent = ReadTransaction then
     begin
      if FQuery.Open then
       FQuery.Close;
      ReadTransaction:=nil;
     end;
    if AComponent = WriteTransaction then
     WriteTransaction:=nil;
   end;  
end;

procedure TUnCustomFibFetcher.Open;
var I, J: Integer;
    TempName: string;
begin
 if not FQuery.Open then
  begin
   FIsFieldsChecked:=false;  //    Fields     Fields FQuery
   if not FQuery.Prepared then
    FQuery.Prepare;
   for I:=0 to FQuery.Params.Count-1 do  //     FQuery
    begin
     TempName:=FQuery.Params[I].Name;
     for J:=0 to FetchDataSet.Params.Count-1 do
      begin
       if CompareText(FQuery.Params[I].Name, FetchDataSet.Params[J].Name) = 0 then
        begin
         FQuery.Params[I].Value:=FetchDataSet.Params[J].Value;
        end;
      end;
    end;
   FQuery.ExecQuery;  //  
  end;
end;

procedure TUnCustomFibFetcher.Close;
begin
 if FQuery.Open then
  FQuery.Close;
 if FieldFechIndex <> nil then
  FreeMem(FieldFechIndex);
 FieldFechIndex:=nil;
end;

procedure TUnCustomFibFetcher.InitializeFields(ADataSet: TDataSet);
var I: Integer;
    Def: TFieldDef;
    J: Integer;
    Fld: TField;
    IsBoolean: boolean;
begin
 if InitializeFromFieldDefData and (FieldDefData.Size > 0) then
  begin   //     TStream.
   FieldDefData.Position:=0;
   LoadFieldDefsFromStream(DataSet.FieldDefs, FieldDefData);
   FIsFieldsChecked:=false;
   HackDataSet(DataSet).FLoadFieldDefsFromStream:=true;
  end
 else
  begin
   if not FQuery.Prepared then
    FQuery.Prepare; //  
   for I:=0 to FQuery.Current.Count-1 do
    begin //    
     // case   
     case (FQuery.Current[I].Data^.sqltype and not 1) of
       SQL_VARYING, SQL_TEXT: begin // 
         TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftString, FQuery.Current[I].Data^.sqllen, true, I+1);
        end;
       SQL_DOUBLE, SQL_D_FLOAT, SQL_FLOAT: begin  //  
         TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftFloat, 0, false, I+1);
        end;
       SQL_INT64: begin   // Int64
         if (FQuery.Current.Vars[I].Scale = 4) or (FQuery.Current.Vars[I].Scale = 2) then
          begin
           Def:=TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftCurrency, 0, false, I+1);
           Def.Precision:=FQuery.Current.Vars[I].Scale;
          end
         else
          begin
           TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftFloat, 0, false, I+1);
          end;
        end;
       SQL_LONG: begin // 
         IsBoolean:=false;
         for J:=0 to DataSet.FieldCount-1 do
          begin
           Fld:=DataSet.Fields[J];
           if CompareText(Fld.FieldName, FQuery.Current.Vars[I].Name) = 0 then
            begin
             IsBoolean:=Fld.DataType = ftBoolean;
             break;
            end
          end;
         if IsBoolean then
          TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftBoolean, 0, false, I+1)
         else
          TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftInteger, 0, false, I+1);
        end;
       SQL_SHORT: begin  //  
         TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftSmallint, 0, false, I+1);
        end;
       SQL_TIMESTAMP: begin  //   
         TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftDateTime, 0, false, I+1);
        end;
       SQL_BLOB: begin  //  
         if (FQuery.Current[I].Data^.sqlsubtype = 0) then
          begin
           TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftMemo, 0, true, I+1);
          end
         else
          begin
           TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftBlob, 0, true, I+1);
          end;
        end;
       SQL_TYPE_TIME: begin // 
         TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftTime, 0, false, I+1);
        end;
       SQL_TYPE_DATE: begin  // 
         TFieldDef.Create(ADataSet.FieldDefs, FQuery.Current.Vars[I].Name,
                 ftDate, 0, false, I+1);
        end;
       else FIBError(feInvalidDataConversion, [nil]);
     end;
     FIsFieldsChecked:=false;
    end;
 end;
end;

procedure TUnCustomFibFetcher.GetNextData(AData: PChar);
var I: Integer;
    Fld: TField;
    IbField: TFIBXSQLVAR;
    IdFld: Integer;
    IbRecord: TFIBXSQLDA;
    IsData: boolean;
begin
 if not FIsFieldsChecked then
  begin //    
   if FieldFechIndex <> nil then
    FreeMem(FieldFechIndex);
   GetMem(FieldFechIndex, DataSet.FieldCount*Sizeof(Integer));
   for I:=0 to DataSet.FieldCount-1 do
    begin
     Fld:=DataSet.Fields[I];
     IdFld:=FQuery.FieldIndex[Fld.FieldName];
     FieldFechIndex[I+1]:=IdFld;
     if (IdFld > -1) then
      begin
       IbField:=FQuery.Fields[IdFld];
       case IbField.Data^.sqltype and not 1  of
        SQL_VARYING: // varchar
         if Fld.DataType <> ftString then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TEXT: // char
         if Fld.DataType <> ftString then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_DOUBLE, SQL_FLOAT, SQL_D_FLOAT, SQL_INT64:  // double precition, numeric, decimal
         if not ((Fld.DataType = ftFloat) or (Fld.DataType =  ftCurrency)) then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_LONG:  // integer
         if not ((Fld.DataType = ftInteger) or (Fld.DataType = ftBoolean)) then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_SHORT: // short
         if Fld.DataType <> ftSmallint then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_BLOB:  // blob
         if not Fld.IsBlob then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TYPE_TIME: // time
         if Fld.DataType <>  ftTime then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TYPE_DATE:  // date
         if Fld.DataType <> ftDate then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TIMESTAMP:  // timestamp
         if Fld.DataType <>  ftDateTime then
          FIBError(feInvalidDataConversion, [nil]);
        else FIBError(feInvalidDataConversion, [nil]);
       end;
      end;
    end;
   FIsFieldsChecked:=true;
  end;
 IbRecord:=FQuery.Current;
 for I:=0 to DataSet.FieldCount-1 do
  begin //   
   Fld:=DataSet.Fields[I];
   IdFld:=FieldFechIndex[I+1];
   if (Fld.FieldKind = fkData) and (IdFld > -1) then
    begin
     IbField:=IbRecord.Vars[IdFld];
     //         -      
     if IbField <> nil then
      begin
       IsData:=false;
       if not Fld.IsBlob then
        begin
         IsData:=not IbField.IsNull;
         PAddData(AData+FetchDataSet.FieldOffset[Fld.FieldNo]+Fld.DataSize)^.IsData:=IsData;
        end;
       if Fld.IsBlob or IsData then
        AFechFuncList[IbField.Data^.sqltype](Fld, IbField.Data, AData + FetchDataSet.FieldOffset[Fld.FieldNo]);
      end
     else
      PAddData(AData+FetchDataSet.FieldOffset[Fld.FieldNo]+Fld.DataSize)^.IsData:=false;
    end;
  end;
 FQuery.Next;
end;

function TUnCustomFibFetcher.GetActive: boolean;
begin
 if FQuery <> nil then
  Result:=FQuery.Open
 else
  Result:=false;
end;

procedure TUnCustomFibFetcher.SetActive(const Value: boolean);
begin
 if Value then
  Open
 else
  Close;
end;

function TUnCustomFibFetcher.GetBof: boolean;
begin
 Result:=FQuery.Bof;
end;

function TUnCustomFibFetcher.GetEof: boolean;
begin
 Result:=FQuery.Eof;
end;

function TUnCustomFibFetcher.GetParamCheck: boolean;
begin
 Result:=FQuery.ParamCheck;
end;

procedure TUnCustomFibFetcher.SetParamCheck(const Value: boolean);
begin
 FQuery.ParamCheck:=Value;
end;

function TUnCustomFibFetcher.GetSQL: TStrings;
begin
 Result:=FQuery.SQL;
end;

procedure TUnCustomFibFetcher.SetSQL(const Value: TStrings);
begin
 FQuery.SQL.Assign(Value);
end;

function TUnCustomFibFetcher.GetReadDatabase: TFIBDatabase;
begin
 if FQuery <> nil then
  Result:=FQuery.Database
 else
  Result:=nil;
end;

procedure TUnCustomFibFetcher.SetReadDatabase(Value: TFIBDatabase);
begin
 if FQuery <> nil then
  FQuery.Database:=Value;
end;

function TUnCustomFibFetcher.GetOptions: TpFIBQueryOptions;
begin
 Result:=FQuery.Options;
end;

procedure TUnCustomFibFetcher.SetOptions(Value: TpFIBQueryOptions);
begin
 FQuery.Options:=Value;
end;

function TUnCustomFibFetcher.GetReadTransaction: TFIBTransaction;
begin
 if FQuery <> nil then
  Result:=FQuery.Transaction
 else
  Result:=nil;
end;

procedure TUnCustomFibFetcher.SetReadTransaction(Value: TFIBTransaction);
begin
 FQuery.Transaction:=Value;
end;

function TUnCustomFibFetcher.GetFieldCount: Word;
begin
 if FQuery <> nil then
  Result:=FQuery.Current.Count
 else
  Result:=0;
end;

// -------------------------   ------------------------------------//
{                                                                             //
                                                                              //
 sqldata -                                                    //
 sqlscale -      10                               //
              ,  sqlscale < 0                         //
                                                                              //
}                                                                             //
procedure FechSQL_VARYING(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);     //
var P: PChar;                                                                 //
    Len: PSmallInt;                                                           //
begin                                                                         //
 Len:=Pointer(IbField^.sqldata);                                              //
 P:=PChar(IbField^.sqldata+Sizeof(SmallInt));                               //
 StrLCopy(Buffer, P, Len^);                                                   //
 FillChar((Buffer+Len^)^, Fld.DataSize-Len^, 0); //  
end;                                                                          //                                                                        //
                                                                              //
procedure FechSQL_TEXT(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);        //
var BoolBuffer: WordBool;                                                     //
begin                                                                         //
 if Fld.DataType = ftBoolean then                                             //
  begin                                                                       //
   BoolBuffer:=UpperCase(PChar((IbField^.sqldata)^)) = 'T';                   //
   move(BoolBuffer, Buffer^, sizeof(Word));                                   //
  end                                                                         //
 else                                                                         //
  begin                                                                       //
   move((IbField^.sqldata)^, Buffer^, IbField^.sqllen);                       //
   Buffer[IbField^.sqllen]:=#0;                                               //
  end;                                                                        //
end;                                                                          //
                                                                              //
procedure FechSQL_DOUBLE(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);      //
var DoubleBuffer: Double;                                                     //
begin                                                                         //
 DoubleBuffer:=PDouble(IbField^.sqldata)^;                                    //
 if IbField^.sqlscale <> 0 then                                               //
 _Degree10Double(DoubleBuffer, IbField^.sqlscale);                            //
 move(DoubleBuffer, Buffer^, sizeof(Double));                                 //
end;                                                                          //
                                                                              //
procedure FechSQL_FLOAT(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);       //
var DoubleBuffer: Double;                                                     //
begin                                                                         //
 DoubleBuffer:=PDouble(IbField^.sqldata)^;                                    //
 _Degree10Double(DoubleBuffer, IbField^.sqlscale);                            //
 move(DoubleBuffer, Buffer^, sizeof(Double));                                 //
end;                                                                          //
                                                                              //
procedure FechSQL_LONG(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);        //
var IntegerBuffer: Longint;                                                   //
    BoolBuffer: WordBool;                                                     //
begin                                                                         //
 if Fld.DataType = ftBoolean then                                             //
  begin                                                                       //
   BoolBuffer:=PLong(IbField^.sqldata)^ <> 0;                                 //
   move(BoolBuffer, Buffer^, sizeof(Word));                                   //
  end                                                                         //
 else                                                                         //
  begin                                                                       //
   IntegerBuffer:=PLong(IbField^.sqldata)^;                                   //
   _Degree10Integer(IntegerBuffer, IbField^.sqlscale);                        //
   move(IntegerBuffer, Buffer^, sizeof(Longint));                             //
 end;                                                                         //
end;                                                                          //
                                                                              //
procedure FechSQL_SHORT(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);       //
var ShortBuffer: SmallInt;                                                    //
begin                                                                         //
 ShortBuffer:=SmallInt(PSmallInt(IbField^.sqldata)^);                         //
 _Degree10Small(ShortBuffer, IbField^.sqlscale);                              //
 move(ShortBuffer, Buffer^, sizeof(SmallInt));                                //
end;                                                                          //
                                                                              //
procedure FechSQL_INT64(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);       //
var DoubleBuffer: Double;                                                     //
begin                                                                         //
 DoubleBuffer:=PDouble(IbField^.sqldata)^;                                    //
 _Degree10Double(DoubleBuffer, IbField^.sqlscale);                            //
 move(DoubleBuffer, Buffer^, sizeof(Double));                                 //
end;                                                                          //
                                                                              //
procedure FechSQL_TIMESTAMP(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);   //
var TimeStampBuffer: TTimeStamp;                                              //
    DateTimeBuffer: TDateTime;                                                //
begin                                                                         //
 with PISC_TIMESTAMP(IbField^.sqldata)^ do                                    //
  begin                                                                       //
   TimeStampBuffer.Date := timestamp_date + IBBuffDateDelta;                  //
   TimeStampBuffer.Time := timestamp_time div 10;                             //
  end;                                                                        //
 DateTimeBuffer := TimeStampToMSecs(TimeStampBuffer) ;                        //
 move(DateTimeBuffer, Buffer^, sizeof(TDateTime));                            //
end;                                                                          //
                                                                              //
procedure FechSQL_TYPE_TIME(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);   //
var DateTimeRecBuffer: TDateTimeRec;                                          //
begin                                                                         //
 DateTimeRecBuffer.Time := PISC_TIME(IbField^.sqldata)^ div 10;               //
 move(DateTimeRecBuffer, Buffer^, sizeof(TDateTimeRec));                      //
end;                                                                          //
                                                                              //
procedure FechSQL_TYPE_DATE(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);   //
var DateTimeRecBuffer: TDateTimeRec;                                          //
begin                                                                         //
 DateTimeRecBuffer.Date := PISC_DATE(IbField^.sqldata)^ + IBBuffDateDelta;    //
 move(DateTimeRecBuffer, Buffer^, sizeof(TDateTimeRec));                      //
end;                                                                          //
                                                                              //
procedure FechSQL_BLOB(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);        //
begin                                                                         //
 PUnBlobQuad(Buffer)^.Blob:=-1;                                               //
end;                                                                          //
                                                                              //
procedure FechERROR(Fld: TField; IbField: PXSQLVAR; Buffer: PChar);           //
begin                                                                         //
 raise TUnDbException.Create(str_Error_Fetch);                                //
end;                                                                          //
//----------------------------------------------------------------------------//

initialization

 for II:=447 to 583 do
  AFechFuncList[II]:=FechERROR;

 AFechFuncList[SQL_VARYING]:=FechSQL_VARYING;
 AFechFuncList[SQL_VARYING+1]:=FechSQL_VARYING;
 AFechFuncList[SQL_TEXT]:=FechSQL_TEXT;
 AFechFuncList[SQL_TEXT+1]:=FechSQL_TEXT;
 AFechFuncList[SQL_DOUBLE]:=FechSQL_DOUBLE;
 AFechFuncList[SQL_DOUBLE+1]:=FechSQL_DOUBLE;
 AFechFuncList[SQL_FLOAT]:=FechSQL_FLOAT;
 AFechFuncList[SQL_FLOAT+1]:=FechSQL_FLOAT;
 AFechFuncList[SQL_LONG]:=FechSQL_LONG;
 AFechFuncList[SQL_LONG+1]:=FechSQL_LONG;
 AFechFuncList[SQL_SHORT]:=FechSQL_SHORT;
 AFechFuncList[SQL_SHORT+1]:=FechSQL_SHORT;
 AFechFuncList[SQL_TIMESTAMP]:=FechSQL_TIMESTAMP;
 AFechFuncList[SQL_TIMESTAMP+1]:=FechSQL_TIMESTAMP;
 AFechFuncList[SQL_BLOB]:=FechSQL_BLOB;
 AFechFuncList[SQL_BLOB+1]:=FechSQL_BLOB;
 AFechFuncList[SQL_D_FLOAT]:=FechSQL_DOUBLE;
 AFechFuncList[SQL_D_FLOAT+1]:=FechSQL_DOUBLE;
 AFechFuncList[SQL_ARRAY]:=FechSQL_BLOB;
 AFechFuncList[SQL_ARRAY+1]:=FechSQL_BLOB;
 AFechFuncList[SQL_QUAD]:=FechSQL_BLOB;
 AFechFuncList[SQL_QUAD+1]:=FechSQL_BLOB;
 AFechFuncList[SQL_TYPE_TIME]:=FechSQL_TYPE_TIME;
 AFechFuncList[SQL_TYPE_TIME+1]:=FechSQL_TYPE_TIME;
 AFechFuncList[SQL_TYPE_DATE]:=FechSQL_TYPE_DATE;
 AFechFuncList[SQL_TYPE_DATE+1]:=FechSQL_TYPE_DATE;
 AFechFuncList[SQL_INT64]:=FechSQL_INT64;
 AFechFuncList[SQL_INT64+1]:=FechSQL_INT64;

end.
