
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnFibFetcher;

interface

 uses SysUtils, Classes, Db, UnCustomDataSet, pFIBQuery, pFIBProps, FIBDatabase,
      UnSQLUpdater, UnFibClasses, FIBQuery, DsgnIntf;

 Type

  TFIBBlobUpdater = class(TSQLBlobUpdater)
   private
    FGetQuery: TpFIBQuery;
    FSetQuery: TpFIBQuery;
    function GetOptions: TpFIBQueryOptions;
    procedure SetOptions(Value: TpFIBQueryOptions);
   protected
    function GetGetSQL: TStrings; override;
    procedure SetGetSQL(Value: TStrings); override;
    function GetSetSQL: TStrings; override;
    procedure SetSetSQL(Value: TStrings); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
   published
    property Options: TpFIBQueryOptions read GetOptions write SetOptions;
  end;

  TUnFibBlobItem = class(TUnSQLBlobItem)
   private
   protected
    function GetOptions: TpFIBQueryOptions;
    procedure SetOptions(Value: TpFIBQueryOptions);
    function CreateBlobUpdater: TBlobUpdater; override;
   public
    procedure Assign(Source: TPersistent); override;
   published
    property Options: TpFIBQueryOptions read GetOptions write SetOptions;
  end;

  TUnFIBFetcher = class(TUnCustomFibFetcher)
   private
    FRefreshQuery: TpFIBQuery;
    FLockQuery: TpFIBQuery;
    FInsertQuery: TpFIBQuery;
    FDeleteQuery: TpFIBQuery;
    FUpdateQuery: TpFIBQuery;
    UpdateNow: boolean;
    RefreshFieldFechIndex: POffsetsArray;
    FIsRefreshFieldsChecked: boolean;
    function GetOptions: TpFIBQueryOptions;
    procedure SetOptions(Value: TpFIBQueryOptions);
    procedure PrepareParams(Query: TpFIBQuery);
    procedure CreateExecProc(SQL: TStrings; Query: TpFIBQuery);
    function BlobByFieldName(const FieldName: string): TFIBBlobUpdater;
   protected
    function GetWriteTransaction: TFIBTransaction; override;
    procedure SetWriteTransaction(Value: TFIBTransaction); override;
    function GetRefreshSQL: TStrings; override;
    procedure SetRefreshSQL(Value: TStrings); override;
    function GetLockSQL: TStrings; override;
    procedure SetLockSQL(Value: TStrings); override;
    function GetNewBlob: TBlobUpdater; override;
    function CreateCollection: TUnUpdateCollection; override;
    function GetWriteDatabase: TFIBDatabase; override;
    procedure SetWriteDatabase(Value: TFIBDatabase); override;
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DoInsert; override;
    procedure DoUpdate; override;
    procedure DoDelete; override;
    procedure DoRefresh;  override;
    procedure DoLock;  override;
    procedure DoGetBlob(Field: TBlobField; var Data: TStream); override;
    procedure DoSetBlob(Field: TBlobField; var Data: TStream); override;
   published
    property Options: TpFIBQueryOptions read GetOptions write SetOptions;
  end;

implementation

 uses ibase, fib;

 Type

    HackDataSet = class(TUnDataSet);

{--------------------------------TFIBBlobUpdater-------------------------------}

constructor TFIBBlobUpdater.Create(AOwner: TComponent);
begin
 inherited;
 FGetQuery:=TpFIBQuery.Create(Self);
 FSetQuery:=TpFIBQuery.Create(Self);
end;

destructor TFIBBlobUpdater.Destroy;
begin
 if FGetQuery.Open then
  FGetQuery.Close;
 if FSetQuery.Open then
  FSetQuery.Close;
 inherited;
end;

procedure TFIBBlobUpdater.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited;
 if AComponent <> nil then
  if Operation = opRemove then
   begin
    if AComponent = FGetQuery.Database then
     FGetQuery.Database:=nil;
    if AComponent = FSetQuery.Database then
     FSetQuery.Database:=nil;
   end;
end;

function TFIBBlobUpdater.GetGetSQL: TStrings;
begin
 Result:=FGetQuery.SQL;
end;

procedure TFIBBlobUpdater.SetGetSQL(Value: TStrings);
begin
 FGetQuery.SQL.Assign(Value);
end;

function TFIBBlobUpdater.GetSetSQL: TStrings;
begin
 Result:=FSetQuery.SQL;
end;

procedure TFIBBlobUpdater.SetSetSQL(Value: TStrings);
begin
 FSetQuery.SQL.Assign(Value);
end;

function TFIBBlobUpdater.GetOptions: TpFIBQueryOptions;
begin
 Result:=FGetQuery.Options;
end;

procedure TFIBBlobUpdater.SetOptions(Value: TpFIBQueryOptions);
begin
 FGetQuery.Options:=Value;
 FSetQuery.Options:=Value;
end;

{---------------------------------TUnFIBUpdater--------------------------------}

constructor TUnFIBFetcher.Create(AOwner: TComponent);
begin
 inherited;
 UpdateNow:=false;
 FRefreshQuery:=TpFIBQuery.Create(Self);
 FLockQuery:=TpFIBQuery.Create(Self);
 FInsertQuery:=TpFIBQuery.Create(Self);
 FDeleteQuery:=TpFIBQuery.Create(Self);
 FUpdateQuery:=TpFIBQuery.Create(Self);
 FRefreshQuery:=TpFIBQuery.Create(Self);
 FLockQuery:=TpFIBQuery.Create(Self);
 FInsertQuery:=TpFIBQuery.Create(Self);
 FDeleteQuery:=TpFIBQuery.Create(Self);
 FUpdateQuery:=TpFIBQuery.Create(Self);
 FIsRefreshFieldsChecked:=false;
 RefreshFieldFechIndex:=nil;
end;

function TUnFIBFetcher.CreateCollection: TUnUpdateCollection;
begin
 Result:=TUnUpdateCollection.Create(TUnFibBlobItem);
 Result.Owner:=Self;
end;

destructor TUnFIBFetcher.Destroy;
begin
 if RefreshFieldFechIndex <> nil then
  FreeMem(RefreshFieldFechIndex);
 RefreshFieldFechIndex:=nil;
 FRefreshQuery.Close;
 FLockQuery.Close;
 FInsertQuery.Close;
 FDeleteQuery.Close;
 FUpdateQuery.Close;
 FRefreshQuery.Close;
 FLockQuery.Close;
 FInsertQuery.Close;
 FDeleteQuery.Close;
 FUpdateQuery.Close;
 inherited;
end;

{procedure TUnFIBFetcher.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited;
 if AComponent <> nil then
  if Operation = opRemove then
   begin
    if AComponent = FRefreshQuery.Database then
     FRefreshQuery.Database:=nil;
    if AComponent = FLockQuery.Database then
     FLockQuery.Database:=nil;
    if AComponent = FInsertQuery.Database then
     FInsertQuery.Database:=nil;
    if AComponent = FDeleteQuery.Database then
     FDeleteQuery.Database:=nil;
    if AComponent = FUpdateQuery.Database then
     FUpdateQuery.Database:=nil;
    if AComponent = FRefreshQuery.Transaction then
     FRefreshQuery.Transaction:=nil;
    if AComponent = FLockQuery.Transaction then
     FLockQuery.Transaction:=nil;
    if AComponent = FInsertQuery.Transaction then
     FInsertQuery.Transaction:=nil;
    if AComponent = FDeleteQuery.Transaction then
     FDeleteQuery.Transaction:=nil;
    if AComponent = FUpdateQuery.Transaction then
     FUpdateQuery.Transaction:=nil;
   end;
end; }

function TUnFIBFetcher.GetNewBlob: TBlobUpdater;
begin
 Result:=TFIBBlobUpdater.Create(Self.Owner);
end;

function TUnFIBFetcher.GetLockSQL: TStrings;
begin
 Result:=FLockQuery.SQL;
end;

procedure TUnFIBFetcher.SetLockSQL(Value: TStrings);
begin
 FLockQuery.SQL.Assign(Value);
end;

function TUnFIBFetcher.GetRefreshSQL: TStrings;
begin
 Result:=FRefreshQuery.SQL;
end;

procedure TUnFIBFetcher.SetRefreshSQL(Value: TStrings);
begin
 FRefreshQuery.SQL.Assign(Value);
end;

function TUnFIBFetcher.GetWriteDatabase: TFIBDatabase;
begin
 Result:=FRefreshQuery.Database;
end;

procedure TUnFIBFetcher.SetWriteDatabase(Value: TFIBDatabase);
begin
 FRefreshQuery.Database:=Value;
 FLockQuery.Database:=Value;
 FInsertQuery.Database:=Value;
 FDeleteQuery.Database:=Value;
 FUpdateQuery.Database:=Value;
end;

function TUnFIBFetcher.GetOptions: TpFIBQueryOptions;
begin
 Result:=FRefreshQuery.Options;
end;

procedure TUnFIBFetcher.SetOptions(Value: TpFIBQueryOptions);
begin
 FRefreshQuery.Options:=Value;
 FLockQuery.Options:=Value;
 FInsertQuery.Options:=Value;
 FDeleteQuery.Options:=Value;
 FUpdateQuery.Options:=Value;
end;

function TUnFIBFetcher.GetWriteTransaction: TFIBTransaction;
begin
 if FRefreshQuery <> nil then
  Result:=FRefreshQuery.Transaction
 else
  Result:=nil;
end;

procedure TUnFIBFetcher.SetWriteTransaction(Value: TFIBTransaction);
begin
 FRefreshQuery.Transaction:=Value;
 FLockQuery.Transaction:=Value;
 FInsertQuery.Transaction:=Value;
 FDeleteQuery.Transaction:=Value;
 FUpdateQuery.Transaction:=Value;
end;

procedure TUnFIBFetcher.PrepareParams(Query: TpFIBQuery);
var I: Integer;
    Fld: TField;   S: string;
begin
 if not Query.Prepared then
  Query.Prepare;
 S:=Query.SQL.Text;
 for I:=0 to Query.Params.Count-1 do
  begin
   Fld:=DataSet.FindField(Query.Params[I].Name);
   if Fld <> nil then
    if not Fld.IsBlob then
     Query.Params[I].AsVariant:=Fld.Value;
  end;
end;

procedure TUnFIBFetcher.DoLock;
begin
 PrepareParams(FLockQuery);
 FLockQuery.ExecProc;
end;

procedure TUnFIBFetcher.CreateExecProc(SQL: TStrings; Query: TpFIBQuery);
var I: Integer;
    TempSql: TStringList;
begin
 TempSql:=TStringList.Create;
 try
  for I:=0 to SQL.Count-1 do
   begin
    if Trim(SQL[I]) = '^' then
     begin
      Query.SQL.Assign(TempSql);
      PrepareParams(Query);
      if Query.IsProc then
       Query.ExecProc
      else
       Query.ExecQuery;
      TempSql.Clear;
     end
    else
     TempSql.Add(SQL[I]);
   end;
  if TempSql.Count > 0 then
   begin
    Query.SQL.Assign(TempSql);
    PrepareParams(Query);
    if Query.IsProc then
     Query.ExecProc
    else
     Query.ExecQuery;
   end;
 finally
  TempSql.Free;
 end;
end;

procedure TUnFIBFetcher.DoInsert;
begin
 CreateExecProc(InsertSQL, FInsertQuery);
end;

procedure TUnFIBFetcher.DoUpdate;
begin
 CreateExecProc(UpdateSQL, FUpdateQuery);
end;

procedure TUnFIBFetcher.DoDelete;
begin
 CreateExecProc(DeleteSQL, FDeleteQuery);
end;

procedure TUnFIBFetcher.DoRefresh;
var I: Integer;
    Fld: TField;
    IbField: TFIBXSQLVAR;
    IbRecord: TFIBXSQLDA;
    IdFld: Integer;
    IsData: boolean;
    Buffer: PChar;
begin
 PrepareParams(FRefreshQuery);
 FRefreshQuery.ExecQuery;
 if not FIsRefreshFieldsChecked then
  begin
   if RefreshFieldFechIndex <> nil then
    FreeMem(RefreshFieldFechIndex);
   GetMem(RefreshFieldFechIndex, DataSet.FieldCount*Sizeof(Integer));
   for I:=0 to DataSet.FieldCount-1 do
    begin
     Fld:=DataSet.Fields[I];
     IdFld:=FRefreshQuery.FieldIndex[Fld.FieldName];
     RefreshFieldFechIndex[I+1]:=IdFld;
     if (IdFld > -1) then
      begin
       IbField:=FRefreshQuery.Fields[IdFld];
       case IbField.Data^.sqltype and not 1  of
        SQL_VARYING: // varchar
         if Fld.DataType <> ftString then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TEXT: // char
         if Fld.DataType <> ftString then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_DOUBLE, SQL_FLOAT, SQL_D_FLOAT, SQL_INT64:  // double precition, numeric, decimal
         if not ((Fld.DataType = ftFloat) or (Fld.DataType =  ftCurrency)) then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_LONG:  // integer
         if not ((Fld.DataType = ftInteger) or (Fld.DataType = ftBoolean)) then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_SHORT: // short
         if Fld.DataType <> ftSmallint then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_BLOB:  // blob
         if not Fld.IsBlob then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TYPE_TIME: // time
         if Fld.DataType <>  ftTime then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TYPE_DATE:  // date
         if Fld.DataType <> ftDate then
          FIBError(feInvalidDataConversion, [nil]);
        SQL_TIMESTAMP:  // timestamp
         if Fld.DataType <>  ftDateTime then
          FIBError(feInvalidDataConversion, [nil]);
        else FIBError(feInvalidDataConversion, [nil]);
       end;
      end;
    end;
   FIsFieldsChecked:=true;
  end;
 IbRecord:=FRefreshQuery.Current;
 Buffer:=HackDataSet(DataSet).ActiveBuffer;
 for I:=0 to DataSet.FieldCount-1 do
  begin //   
   Fld:=DataSet.Fields[I];
   IdFld:=RefreshFieldFechIndex[I+1];
   if (Fld.FieldKind = fkData) and (IdFld > -1) then
    begin
     IbField:=IbRecord.Vars[IdFld];
     //         -      
     if IbField <> nil then
      begin
       IsData:=false;
       if not Fld.IsBlob then
        begin
         IsData:=not IbField.IsNull;
         PAddData(Buffer+FetchDataSet.FieldOffset[Fld.FieldNo]+Fld.DataSize)^.IsData:=IsData;
        end;
       if Fld.IsBlob or IsData then
        AFechFuncList[IbField.Data^.sqltype](Fld, IbField.Data, Buffer + FetchDataSet.FieldOffset[Fld.FieldNo]);
      end
     else
      PAddData(Buffer+FetchDataSet.FieldOffset[Fld.FieldNo]+Fld.DataSize)^.IsData:=false;
    end;
  end;
end;

function TUnFIBFetcher.BlobByFieldName(const FieldName: string): TFIBBlobUpdater;
var I: Integer;
begin
 Result:=nil;
 for I:=0 to BlobsCollection.Count-1 do
  begin
   if CompareText(TUnFibBlobItem(BlobsCollection.Items[I]).FBlobUpdater.FieldName, FieldName) = 0 then
    begin
     Result:=TUnFibBlobItem(BlobsCollection.Items[I]).FBlobUpdater as TFIBBlobUpdater;
     Break;
    end;
  end;
end;

procedure TUnFIBFetcher.DoGetBlob(Field: TBlobField; var Data: TStream);
var S: TFIBBlobUpdater;
    P: TFIBXSQLVAR;
begin
 S:=BlobByFieldName(Field.FieldName);
 if S <> nil then
  begin
   if Field.DataSet.State = dsEdit then
    begin
     S.FGetQuery.Database:=WriteDatabase;
     S.FGetQuery.Transaction:=WriteTransaction;
    end
   else
    begin
     S.FGetQuery.Database:=ReadDatabase;
     S.FGetQuery.Transaction:=ReadTransaction;
    end;
   PrepareParams(S.FGetQuery);
   S.FGetQuery.ExecQuery;
   Data.Size:=0;
   P:=S.FGetQuery.FN(Field.FieldName);
   if P <> nil then
    P.SaveToStream(Data);
  end;
end;

procedure TUnFIBFetcher.DoSetBlob(Field: TBlobField; var Data: TStream);
var S: TFIBBlobUpdater;
    P: TFIBXSQLVAR;
begin
 S:=BlobByFieldName(Field.FieldName);
 if S <> nil then
  begin
   S.FGetQuery.Database:=WriteDatabase;
   S.FGetQuery.Transaction:=WriteTransaction;
   PrepareParams(S.FSetQuery);
   P:= S.FSetQuery.ParamByName(Field.FieldName);
   if P <> nil then
    begin
     Data.Position:=0;
     P.LoadFromStream(Data);
     S.FSetQuery.ExecQuery;
    end;
  end;
end;

{-------------------------------TUnFibBlobItem---------------------------------}

procedure TUnFibBlobItem.Assign(Source: TPersistent);
begin
 if Source is TUnFibBlobItem then
  FBlobUpdater:=TUnFibBlobItem(Source).FBlobUpdater
 else
  Inherited Assign(Source);
end;

function TUnFibBlobItem.GetOptions: TpFIBQueryOptions;
begin
 if FBlobUpdater <> nil then
  Result:=(FBlobUpdater as TFIBBlobUpdater).Options
 else
  Result:=[];
end;

procedure TUnFibBlobItem.SetOptions(Value: TpFIBQueryOptions);
begin
 if FBlobUpdater <> nil then
  (FBlobUpdater as TFIBBlobUpdater).Options:=Value;
end;

function TUnFibBlobItem.CreateBlobUpdater: TBlobUpdater;
begin
 Result:=TFIBBlobUpdater.Create((Collection as TUnUpdateCollection).Owner.Owner);
end;

initialization

 RegisterClass(TFIBBlobUpdater);

finalization

 UnRegisterClass(TFIBBlobUpdater);

end.
