
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnFileServices;

{$I Un.INC}                                              

interface

 uses Windows, SysUtils, Classes {$IFDEF Un_UDB} , UnZLib {$ENDIF};

 Type

  TUDBTempFile = class(TStream)
   private
    FTempStream: TStream;
    FIsMemorytream: boolean;
    FModified: boolean;
   protected
    procedure SetSize(NewSize: Longint); override;
    procedure RecreateStream;
   public
    constructor Create;
    destructor Destroy; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Read(var Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    property Modified: boolean read FModified write FModified;
  end;

  TUnTempFile = class(TFileStream)
   private
    FFileName: string;
    FAutoDelete: boolean;
   public
    constructor Create(const FileName: string; Mode: Word);
    destructor Destroy; override;
    property FileName: string read FFileName;
    property AutoDelete: boolean read FAutoDelete write FAutoDelete;
  end;

  function GetTempDirectory: string;
  function GetTempFile: TStream;
  function GetTempStream: TStream;
  function GetTempStreamAsName(const FileName: string): TStream;
{$IFDEF Un_UDB}
  function PackStream(const Str: TStream; CompressionLevel: TCompressionLevel): TStream;
  function UnPackStream(const Str: TStream): TStream;
{$ENDIF}

  Var TempDirectory: string;

  const

  TempPrefix = 'udb_';
  MAX_MEMORY_STREAM = 512000;
  //MAX_MEMORY_STREAM = 0;

implementation

 uses syncobjs;

{var CS: TCriticalSection;
    F: Integer; }

{$IFDEF Un_UDB}

function PackStream(const Str: TStream; CompressionLevel: TCompressionLevel): TStream;
var Comp: TCompressionStream;
    SizeStream: Integer;
begin
 Result:=GetTempStream;
 SizeStream:=Str.Size;
 Result.Write(SizeStream, Sizeof(SizeStream));
 if SizeStream > 0 then
  begin
   Comp:=TCompressionStream.Create(CompressionLevel, Result);
   try
    Str.Position:=0;
    Comp.CopyFrom(Str, SizeStream);
   finally
    Comp.Free;
   end;
   Result.Position:=0;
  end;
end;

function UnPackStream(const Str: TStream): TStream;
var Comp: TDecompressionStream;
    SizeStream: Integer;
begin
 Result:=GetTempStream;
 Str.Read(SizeStream, Sizeof(Integer));
 if SizeStream > 0 then
  begin
   Comp:=TDecompressionStream.Create(Str);
   try
    Result.CopyFrom(Comp, SizeStream);
   finally
    Comp.Free;
   end;
  end;
end;

{$ENDIF}

function GetTempDirectory: string;
var TempPath: string;
begin
 SetLength(TempPath, MAX_PATH);
 GetTempPath(MAX_PATH, PChar(TempPath));
 Result:=Trim(string(PChar(TempPath)));
end;

function GetTempStream: TStream;
begin
 {CS.Enter;
 try}
  Result:=TUDBTempFile.Create;
 {finally
  CS.Leave;
 end;}
end;

function GetTempFile: TStream;
var TempPath: string;
    TempFileName: string;
begin
 {if F >= 2147483647 then
  F:=0;
 inc(F); }

 SetLength(TempFileName, MAX_PATH);
 FillChar(TempFileName[1], MAX_PATH, #0);
 TempPath:=TempDirectory;
 //TempFileName:=TempPath+TempPrefix+IntToStr(F)+'.tmp';
 GetTempFileName(PChar(TempPath),
                 PChar(TempPrefix),
                 0,
                 PChar(TempFileName));
 Result:=TUnTempFile.Create(Trim(TempFileName), fmCreate);
 //Result:=TUnTempFile.Create(TempFileName, fmCreate);
end;

function GetTempStreamAsName(const FileName: string): TStream;
var sr: TSearchRec;
begin
 if FindFirst(FileName, faAnyFile, sr) = 0 then
  begin
   Result:=TUnTempFile.Create(FileName, fmOpenReadWrite);
   TUnTempFile(Result).AutoDelete:=false;
  end
 else
  Result:=TUnTempFile.Create(FileName, fmCreate);
end;

{ TUnTempFile }

constructor TUnTempFile.Create(const FileName: string; Mode: Word);
begin
 FFileName:=FileName;
 FAutoDelete:=true;
 inherited Create(FileName, Mode);
end;

destructor TUnTempFile.Destroy;
begin
 inherited; 
 if FAutoDelete then
  if FileExists(FFileName) then       
   DeleteFile(FFileName);
end;

{TUDBTempFile}

constructor TUDBTempFile.Create;
begin
 FTempStream:=TMemoryStream.Create;
 FIsMemorytream:=true;
 FModified:=false;
end;

destructor TUDBTempFile.Destroy;
begin
 FTempStream.Free;
 inherited;
end;

procedure TUDBTempFile.SetSize(NewSize: Longint);
begin
 if NewSize < FTempStream.Position then
  FTempStream.Position:=NewSize;
 FTempStream.Size:=NewSize;
 FModified:=true;
end;

procedure TUDBTempFile.RecreateStream;
var OldStream: TStream;
    OldPosition: Integer;
begin
 OldStream:=FTempStream;
 OldPosition:=FTempStream.Position;
 FTempStream:=GetTempFile;
 OldStream.Position:=0;
 FTempStream.CopyFrom(OldStream, OldStream.Size);
 FTempStream.Position:=OldPosition;
 OldStream.Free;
 FIsMemorytream:=false;
end;

function TUDBTempFile.Write(const Buffer; Count: Longint): Longint;
begin
 if FIsMemorytream and ((FTempStream.Position + Count) > MAX_MEMORY_STREAM) then
  RecreateStream;
 Result:=FTempStream.Write(Buffer, Count);
 FModified:=Count > 0;
end;

function TUDBTempFile.Read(var Buffer; Count: Longint): Longint;
begin
 Result:=FTempStream.Read(Buffer, Count);
end;

function TUDBTempFile.Seek(Offset: Longint; Origin: Word): Longint;
begin
 Result:=FTempStream.Seek(Offset, Origin);
end;

initialization

 TempDirectory:=GetTempDirectory;

 {CS:=TCriticalSection.Create;
 F:=0;

finalization

 CS.Free; }

end.
