unit UnParametrEditor;

{$I Un.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
  {$IFDEF Un_D6}
  DesignIntf, DesignEditors,
{$ELSE}
  DsgnIntf,
{$ENDIF}
  DbTables, UnFetchTable, Db;

type
  TfParamsEditor = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Panel2: TPanel;
    Label1: TLabel;
    eValue: TEdit;
    cbTypeData: TComboBox;
    Label2: TLabel;
    Panel3: TPanel;
    lbParams: TListBox;
    Button4: TButton;
    Label3: TLabel;
    eName: TEdit;
    cbTypeParam: TComboBox;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lbParamsClick(Sender: TObject);
    procedure cbTypeDataChange(Sender: TObject);
    procedure eValueExit(Sender: TObject);
    procedure eValueKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure eNameExit(Sender: TObject);
    procedure cbTypeParamChange(Sender: TObject);
    procedure eNameKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
  private
    FParams: TParams;
    ItemIndex: Integer;
    procedure RefreshParamsImage;
  public
    { Public declarations }
  end;

  TUnParametrEditor = class(TPropertyEditor)
   private
   protected
   public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure SetValue(const Value: string); override;
    function GetValue: string; override;
  end;

var
  fParamsEditor: TfParamsEditor;

implementation

{$R *.DFM}

procedure TfParamsEditor.RefreshParamsImage;
var I: Integer;
begin
 lbParams.Clear;
 for I:=0 to FParams.Count-1 do
  begin
   lbParams.Items.Add(FParams.Items[I].Name);
   eName.Text:=FParams.Items[I].Name;
   eValue.Text:=FParams.Items[I].Value;
   cbTypeParam.ItemIndex:=Byte(FParams.Items[I].ParamType);
   cbTypeData.ItemIndex:=Byte(FParams.Items[I].DataType);
  end;
end;

procedure TfParamsEditor.lbParamsClick(Sender: TObject);
begin
 ItemIndex:=lbParams.ItemIndex;
 if ItemIndex >=0 then
  begin
   cbTypeData.ItemIndex:=Byte(FParams.Items[ItemIndex].DataType);
   eName.Text:=FParams.Items[ItemIndex].Name;
   eValue.Text:=FParams.Items[ItemIndex].Value;
   cbTypeParam.ItemIndex:=Byte(FParams.Items[ItemIndex].ParamType);
   cbTypeData.ItemIndex:=Byte(FParams.Items[ItemIndex].DataType);
  end
 else
  begin
   eName.Text:='';
   eValue.Text:='';
   cbTypeParam.ItemIndex:=-1;
   cbTypeData.ItemIndex:=-1;
  end;
end;

procedure TfParamsEditor.cbTypeDataChange(Sender: TObject);
begin
 if ItemIndex >=0 then
  begin
   FParams.Items[ItemIndex].DataType:=TFieldType(cbTypeData.ItemIndex);
   RefreshParamsImage;
  end;
end;

procedure TfParamsEditor.eValueExit(Sender: TObject);
begin
 if ItemIndex >=0 then
  begin
   if Trim(eValue.Text) <> '' then
    begin
     case FParams.Items[ItemIndex].DataType of
      ftSmallint: FParams.Items[ItemIndex].AsSmallInt:=StrToInt(eValue.Text);
      ftInteger: FParams.Items[ItemIndex].AsInteger:=StrToInt(eValue.Text);
      ftWord: FParams.Items[ItemIndex].AsWord:=StrToInt(eValue.Text);
      ftBoolean: begin
        if CompareText(eValue.Text, 'true') = 0 then
         FParams.Items[ItemIndex].AsBoolean:=true
        else
         FParams.Items[ItemIndex].AsBoolean:=false;
       end;
      ftFloat: FParams.Items[ItemIndex].AsFloat:=StrToFloat(eValue.Text);
      ftCurrency: FParams.Items[ItemIndex].AsCurrency:=StrToFloat(eValue.Text);
      ftBCD: FParams.Items[ItemIndex].AsBCD:=StrToFloat(eValue.Text);
      ftDate: FParams.Items[ItemIndex].AsDate:=StrToDate(eValue.Text);
      ftTime: FParams.Items[ItemIndex].AsTime:=StrToTime(eValue.Text);
      ftDateTime: FParams.Items[ItemIndex].AsDateTime:=StrToDateTime(eValue.Text);
      ftString: FParams.Items[ItemIndex].AsString:=eValue.Text;
      else eValue.Text:='';
     end;
    end;
   RefreshParamsImage;
  end;
end;

procedure TfParamsEditor.eValueKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Key = VK_RETURN then
  eValueExit(eValue);
end;

procedure TfParamsEditor.Button2Click(Sender: TObject);
begin
 FParams.CreateParam(ftUnknown, 'NewParametr', ptUnknown);
 RefreshParamsImage;
 ItemIndex:=lbParams.Items.Count-1;
 lbParamsClick(lbParams);
 eName.SetFocus;
end;

procedure TfParamsEditor.Button3Click(Sender: TObject);
var Old: Integer;
begin
 if ItemIndex >= 0 then
  begin
   Old:=ItemIndex;
   FParams.RemoveParam(FParams.Items[ItemIndex]);
   RefreshParamsImage;
   if Old > lbParams.Items.Count - 1 then
    Old:=lbParams.Items.Count - 1;
   lbParams.ItemIndex:=Old;
   lbParamsClick(lbParams);
  end;
end;

procedure TfParamsEditor.eNameExit(Sender: TObject);
begin
 if ItemIndex >=0 then
  begin
   FParams.Items[ItemIndex].Name:=eName.Text;
   RefreshParamsImage;
  end;
end;

procedure TfParamsEditor.eNameKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Key = VK_RETURN then
  eNameExit(eName);
end;

procedure TfParamsEditor.cbTypeParamChange(Sender: TObject);
begin
 if ItemIndex >=0 then
  begin
   FParams.Items[ItemIndex].ParamType:=TParamType(cbTypeParam.ItemIndex);
   RefreshParamsImage;
  end;
end;

procedure TfParamsEditor.FormCreate(Sender: TObject);
begin
 ItemIndex:=-1;
 FParams:=TParams.Create;
 cbTypeData.Clear;
 cbTypeData.Items.Add('ftUnknown');
 cbTypeData.Items.Add('ftString');
 cbTypeData.Items.Add('ftSmallint');
 cbTypeData.Items.Add('ftInteger');
 cbTypeData.Items.Add('ftWord');
 cbTypeData.Items.Add('ftBoolean');
 cbTypeData.Items.Add('ftFloat');
 cbTypeData.Items.Add('ftCurrency');
 cbTypeData.Items.Add('ftBCD');
 cbTypeData.Items.Add('ftDate');
 cbTypeData.Items.Add('ftTime');
 cbTypeData.Items.Add('ftDateTime');
 cbTypeData.Items.Add('ftBytes');
 cbTypeData.Items.Add('ftVarBytes');
 cbTypeData.Items.Add('ftAutoInc');
 cbTypeData.Items.Add('ftBlob');
 cbTypeData.Items.Add('ftMemo');
 cbTypeData.Items.Add('ftGraphic');
 cbTypeData.Items.Add('ftFmtMemo');
 cbTypeData.Items.Add('ftParadoxOle');
 cbTypeData.Items.Add('ftDBaseOle');
 cbTypeData.Items.Add('ftTypedBinary');
 cbTypeData.Items.Add('ftCursor');
{$IFDEF UN_D4}
 cbTypeData.Items.Add('ftFixedChar');
 cbTypeData.Items.Add('ftWideString');
 cbTypeData.Items.Add('ftLargeInt');
 cbTypeData.Items.Add('ftADT');
 cbTypeData.Items.Add('ftArray');
 cbTypeData.Items.Add('ftReference');
 cbTypeData.Items.Add('ftDataSet');
{$ENDIF}
{$IFDEF UN_D5}
 cbTypeData.Items.Add('ftOraBlob');
 cbTypeData.Items.Add('ftOraClob');
 cbTypeData.Items.Add('ftVariant');
 cbTypeData.Items.Add('ftInterface');
 cbTypeData.Items.Add('ftIDispatch');
 cbTypeData.Items.Add('ftGuid');
{$ENDIF}
 cbTypeParam.Clear;
 cbTypeParam.Items.Add('ptUnknown');
 cbTypeParam.Items.Add('ptInput');
 cbTypeParam.Items.Add('ptOutput');
 cbTypeParam.Items.Add('ptInputOutput');
 cbTypeParam.Items.Add('ptResult');
 lbParams.OnClick:=lbParamsClick;
 cbTypeParam.OnChange:=cbTypeParamChange;
 cbTypeData.OnChange:=cbTypeDataChange;
end;

procedure TfParamsEditor.FormDestroy(Sender: TObject);
begin
 FParams.Free;
end;

procedure TfParamsEditor.FormActivate(Sender: TObject);
begin
 if ItemIndex < 0 then
  if lbParams.Items.Count > 0 then
   lbParams.ItemIndex:=0;
 lbParamsClick(lbParams);
end;

{------------------------------TUnParametrEditor-------------------------------}

function TUnParametrEditor.GetAttributes: TPropertyAttributes;
begin
 Result:=[paDialog];
end;

procedure TUnParametrEditor.Edit;
var F: TfParamsEditor;
    Addr: Integer;
begin
 F:=TfParamsEditor.Create(nil);
 try
  Addr:=GetOrdValueAt(0);
  F.FParams.Assign(TParams(Pointer(Addr)));
  F.RefreshParamsImage;
  if F.ShowModal = mrOk then
   SetValue(IntToStr(Integer(F.FParams)));
 finally
  F.Free;
 end;
end;

procedure TUnParametrEditor.SetValue(const Value: string);
var Addr: Integer;
begin
 if Trim(Value) <> '' then
  begin
   Addr:=GetOrdValueAt(0);
   if Addr <> 0 then
    begin
     TParams(Pointer(Addr)).Clear;
     TParams(Pointer(Addr)).Assign(TParams(Pointer(StrToInt(Value))));
     Modified;
    end;
   end;
end;

function TUnParametrEditor.GetValue: string;
begin
 Result:='(Params)';
end;

end.
