
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnRecordSet;

{$I Un.INC}                                              

interface

 uses sysutils, Classes, Db;

 Type

   TUnBatchState = (ubs_None, ubs_Insert, ubs_Edit, ubs_Delete);

   //   Blob
   TUnBlobQuad = packed record
    Blob: Integer;
   end;
   PUnBlobQuad = ^TUnBlobQuad;

   // ,  
   TUnBookmarkInfo = packed record
    BookmarkData: Integer;
    BookmarkFlag: TBookmarkFlag;
   end;
   PUnBookmarkInfo = ^TUnBookmarkInfo;

   // ,    .      
   TUnRecordData = packed record
    Id: Integer;        // 
    Position: Integer;  //   
    BatchState: TUnBatchState;
   end;
   PUnRecordData = ^TUnRecordData;  //    

   TUnCustomRecordSet = class(TObject)
    private
     FDataSet: TDataSet;
    protected
     FListId: TList;
     FData: TList;
     FListBlobs: TList;
     FNextId: Integer;
     FEmptyBlob: TStream;
     function GetItem(Index: Integer): PUnRecordData;
     function GetCapasity: Integer;
     procedure SetCapasity(Value: Integer); virtual;
     function GetCount: Integer;
     procedure DeleteItem(P: Pointer); virtual; abstract;
     function CreateItem: Pointer; virtual; abstract;
     function GetBlobStream(Index: Integer): TStream;
     function GetBufferSize: Integer; virtual; abstract;
     procedure SetDataSet(Value: TDataSet);
     function GetBlobSize: Integer; virtual;
    public
     constructor Create;
     destructor Destroy; override;
     procedure Clear; virtual;
     procedure ClearData; virtual; abstract;
     procedure Open; virtual; abstract;
     procedure Pack; virtual; abstract;
     procedure RecalckPosition;
     procedure Delete(Index: Integer);
     function AddNewItem(Buffer: Pointer; Position: Integer): PUnRecordData; virtual;
     procedure CopyRecordToBuffer(Item: Pointer; Buffer: PChar); virtual; abstract;
     procedure CopyBufferToRecord(Item: Pointer; Buffer: PChar); virtual; abstract;
     function GetBlob: Integer; virtual; abstract;
     function FindRecordByID(ID: Integer): PUnRecordData;
     procedure Exchange(Index1, Index2: Integer);
     function GetStream(AsCopy: boolean): TStream; virtual; abstract; 
     procedure OpenFromStream(Str: TStream); virtual; abstract;
     function AddData(const Str: TStream): Integer; virtual; abstract;
     procedure SaveData(const Str: TStream);  virtual; abstract;
     property Item[Index: Integer]: PUnRecordData read GetItem; default;
     property Count: Integer read GetCount;
     property Capasity: Integer read GetCapasity write SetCapasity;
     property DataSet: TDataSet read FDataSet write SetDataSet;
     property Blob[Index: Integer]: TStream read GetBlobStream;
     property BufferSize: Integer read GetBufferSize;
     property BlobSize: Integer read GetBlobSize;
   end;

   TUnCustomRecordSetDispatcher = class(TComponent)
    private
     FCapasity: Integer;
    public
     function GetRecordSet(DataSet: TDataSet): TUnCustomRecordSet; virtual; abstract;
    published
     property Capasity: Integer read FCapasity write FCapasity;
   end;
   
 {var

  SizesFields: array[0..22] of Byte;   //       }
  
procedure SaveFieldDefsToStream(FieldDefs: TFieldDefs; Str: TStream);
procedure LoadFieldDefsFromStream(FieldDefs: TFieldDefs; Str: TStream);
procedure SaveFieldsDefToStream(DataSet: TDataSet; Str: TStream);

implementation

 uses UnConstants, UnCustomDataSet;

procedure SaveFieldDefsToStream(FieldDefs: TFieldDefs; Str: TStream);
var I: Integer;
    FieldDef: TFieldDef;
begin
 SaveInt(FieldDefs.Count, Str);
 for I:=0 to FieldDefs.Count-1 do
  begin
   FieldDef:=FieldDefs[I];
   SaveString(FieldDef.Name, Str);
   SaveByte(Byte(FieldDef.DataType), Str);
   SaveInt(FieldDef.Size, Str);
   SaveInt(FieldDef.Precision, Str);
   SaveBool(FieldDef.InternalCalcField, Str);
   SaveBool(FieldDef.Required, Str);
  end;
end;

procedure LoadFieldDefsFromStream(FieldDefs: TFieldDefs; Str: TStream);
var I, Count: Integer;
    Name: string;
    DataType: TFieldType;
    Size: Integer;
    Precition: Integer;
    CalckField: boolean;
    Required: boolean;
    Def: TFieldDef;
begin
 if Str.Size > 0 then
  begin
   Count:=LoadInt(Str);
   for I:=0 to Count-1 do
    begin
     Name:=LoadString(Str);
     DataType:=TFieldType(LoadByte(Str));
     Size:=LoadInt(Str);
     Precition:=LoadInt(Str);
     CalckField:=LoadBool(Str);
     Required:=LoadBool(Str);
     Def:=TFieldDef.Create(FieldDefs, Name,
                    DataType, Size, Required, I+1);
     Def.Precision:=Precition;
     Def.InternalCalcField:=CalckField;
    end;
   end;
end;

procedure SaveFieldsDefToStream(DataSet: TDataSet; Str: TStream);
var I: Integer;
    Fld: TField;
begin
 SaveInt(DataSet.FieldCount, Str);
 for I:=0 to DataSet.FieldCount-1 do
  begin
   Fld:=DataSet.Fields[I];
   SaveString(Fld.FieldName, Str);
   SaveByte(Byte(Fld.DataType), Str);
   SaveInt(Fld.Size, Str);
   if Fld.DataType in [ftFloat, ftCurrency] then
    SaveInt(TFloatField(Fld).Precision, Str)
   else
    SaveInt(0, Str);
   SaveBool(Fld.Calculated, Str);
   SaveBool(Fld.Required, Str);
   //SaveInt(Fld.FieldNo, Str);
  end;
end;

{-----------------------------TUnCustomRecordSet-------------------------------}

constructor TUnCustomRecordSet.Create;
begin
 inherited;
 FData:=TList.Create;
 FListId:=TList.Create;
 FNextId:=Low(Integer);
 FListBlobs:=TList.Create;
 FDataSet:=nil;
 FEmptyBlob:=TMemoryStream.Create;
end;

destructor TUnCustomRecordSet.Destroy;
begin
 Clear;
 FData.Free;
 FListId.Free;
 FListBlobs.Free;
 FNextId:=0;
 FEmptyBlob.Free;
 inherited;
end;

function TUnCustomRecordSet.GetBlobSize: Integer;
begin
 Result:=Sizeof(TUnBlobQuad);
end;

procedure TUnCustomRecordSet.SetDataSet(Value: TDataSet);
begin
 if Value is TUnDataSet then
  FDataSet:=Value
 else
  raise Exception.Create('DataSet is not TUnDataSet !');
end;

procedure TUnCustomRecordSet.Clear;
var I: Integer;
begin
 for I:=0 to FData.Count-1 do
  DeleteItem(FData[I]);
 FData.Clear;
 FListId.Clear;
 for I:=0 to FListBlobs.Count-1 do
  TObject(FListBlobs[I]).Free;
 FListBlobs.Clear;
end;

function TUnCustomRecordSet.GetBlobStream(Index: Integer): TStream;
begin
 if (Index >= 0) and (Index < FListBlobs.Count) then
  Result:=FListBlobs[Index]
 else
  begin
   FEmptyBlob.Size:=0;
   Result:=FEmptyBlob;
  end;
end;

procedure TUnCustomRecordSet.RecalckPosition;
var I: Integer;
begin
 for I:=0 to FData.Count-1 do
  PUnRecordData(FData[I])^.Position:=I;
end;

procedure TUnCustomRecordSet.Delete(Index: Integer);
var Id: Integer;
begin
 Id:=PUnRecordData(FData[Index])^.Id;
 DeleteItem(FData[Index]);
 FData.Delete(Index);
 FListId[Id-Low(Integer)]:=nil;
end;

function TUnCustomRecordSet.FindRecordByID(ID: Integer): PUnRecordData;
var Index: Integer;
begin
 Index:=ID-Low(Integer);
 if ( Index >= 0 ) and ( Index < FListId.Count ) then
  Result:=FListId[ID-Low(Integer)]
 else
  Result := nil;  //  ,  Id    
end;

function TUnCustomRecordSet.GetItem(Index: Integer): PUnRecordData;
begin
 Result:=FData[Index];
end;

function TUnCustomRecordSet.GetCapasity: Integer;
begin
 Result:=FData.Capacity;
end;

procedure TUnCustomRecordSet.SetCapasity(Value: Integer);
begin
 FData.Capacity:=Value;
 FListId.Capacity:=Value;
end;

function TUnCustomRecordSet.GetCount: Integer;
begin
 Result:=FData.Count;
end;

function TUnCustomRecordSet.AddNewItem(Buffer: Pointer; Position: Integer): PUnRecordData;
begin
 Result:=PUnRecordData(CreateItem);
 FListId.Add(Result);  //     Id
 Result^.Id:=FNextId;
 Inc(FNextId, 1);
 if FData.Count = Position then    
  Position:=FData.Add(Result)  //   
 else
  FData.Insert(Position, Result);  // 
 Result^.Position:=Position;
 Result^.BatchState:=ubs_None;
end;

procedure TUnCustomRecordSet.Exchange(Index1, Index2: Integer);
var T: Pointer;
begin
 T := FData[Index1];
 FData[Index1] := FData[Index2];
 PUnRecordData(FData[Index1])^.Position:=Index1;
 FData[Index2] := T;
 PUnRecordData(FData[Index2])^.Position:=Index2;
end;

end.
