
          //--------------------------------------------//
          //           .         //
          //            !           //
          //         Copiryght@ 1996-2004.              //
          //         Programmer Bannikov N.A.           //
          //         Universal Database Server 6.0      //
          //         STIKRIZ Technology.                //
          //--------------------------------------------//

unit UnStreamRecordSet;

{$I Un.INC}                                              

interface

 uses Classes, Db,
{$IFDEF Un_D6}
 DesignIntf,
{$ELSE}
 DsgnIntf,
{$ENDIF}
 UnRecordSet, UnFileServices, UnCustomDataSet;

 Type

  TGetSetTempFile = procedure(Sender: TObject; var TempFile: TStream) of Object;
  TCopyBufferRecordSet = procedure(Offset: Integer; Buffer: PChar) of Object;

  // ,    .
  TUnStrRecordData = packed record
   Id: Integer;        // 
   Position: Integer;  //   
   BatchState: TUnBatchState;
   Offset: Longint;    //   
  end;
  PUnStrRecordData = ^TUnStrRecordData;  //    

  TUnStreamRecordSetDispatcher = class;

  TUnStreamRecordSet = class(TUnCustomRecordSet)
   private
    FStream: TStream;           // ,     
    FCashMode: boolean;
    FBufferSize: Integer;       //    
    FNextOffset: Longint;       //    
    FCash: PChar;               //  
    FCashOfset: Longint;        // ,      
    FCashWriten: boolean;       //     ?
    FCashSize: Integer;         //     
    FRecordSize: Integer;       //  
    FDispetcher: TUnStreamRecordSetDispatcher;
    CopyBufferToRecordMethod: TCopyBufferRecordSet; //   
    CopyRecordToBufferMethod: TCopyBufferRecordSet; //   
    procedure CopyBufferToRecordWizBubber(Offset: Integer; Buffer: PChar);
    procedure CopyRecordToBufferWizBuffer(Offset: Integer; Buffer: PChar);
    procedure CopyBufferToRecordBubber(Offset: Integer; Buffer: PChar);
    procedure CopyRecordToBufferBuffer(Offset: Integer; Buffer: PChar);
    procedure ResetCash;
    procedure ClearCash;
    procedure CheckFileOffset(AOffset: Longint);
    procedure InternalOpen;
    procedure WriteStream(Value: TStream);
    procedure WriteBufferSize(Value: Integer);
    procedure WriteCashMode(Value: Boolean);
   protected
    procedure DeleteItem(P: Pointer); override;
    function CreateItem: Pointer; override;
    function GetBufferSize: Integer; override;
   public
    constructor Create;
    destructor Destroy; override;
    procedure ClearData; override;
    procedure Pack; override;
    function GetBlob: Integer; override;
    function AddNewItem(Buffer: Pointer; Position: Integer): PUnRecordData; override;
    procedure CopyRecordToBuffer(Item: Pointer; Buffer: PChar); override;
    procedure CopyBufferToRecord(Item: Pointer; Buffer: PChar); override;
    function GetStream(AsCopy: boolean): TStream; override;
    procedure Open; override;
    procedure OpenFromStream(Str: TStream); override;
    function AddData(const Str: TStream): Integer; override;
    procedure SaveData(const Str: TStream); override;
    property Stream: TStream read FStream Write WriteStream;
    property BufferSize: integer read FBufferSize Write WriteBufferSize;
    property CashMode: boolean read FCashMode Write WriteCashMode;
  end;

  TUnStreamRecordSetDispatcher = class(TUnCustomRecordSetDispatcher)
   private
    FBufferSize: Integer;
    FCashMode: boolean;
   public
    function GetRecordSet(DataSet: TDataSet): TUnCustomRecordSet; override;
   published
    property BufferSize: Integer read FBufferSize write FBufferSize;
    property CashMode: boolean  read FCashMode write FCashMode;
  end;

implementation

 uses UnConstants, sysutils, UnMessages;

{-----------------------------TUnStreamRecordSet-------------------------------}

constructor TUnStreamRecordSet.Create;
begin
 inherited;
 FCash:=nil;
end;

destructor TUnStreamRecordSet.Destroy;
begin
 FStream.Free;
 FStream:=nil;
 ClearCash;
 inherited;
end;

procedure TUnStreamRecordSet.ClearData;
begin
 if FStream <> nil then
  FStream.Size:=0;
 FNextOffset:=0;
 FCashOfset:=0;
 FCashWriten:=false;
end;

procedure TUnStreamRecordSet.ClearCash;
begin
 if FCash <> nil then
  FreeMem(FCash, FCashSize);
 FCash:=nil;
end;

function TUnStreamRecordSet.GetBlob: Integer;
begin
 Result:=FListBlobs.Add(GetTempStream);
end;

procedure TUnStreamRecordSet.Open;
begin
 FNextId:=Low(Integer);
 FRecordSize:=DataSet.RecordSize+Sizeof(TUnBookmarkInfo);
 FNextOffset:=FStream.Size;
 InternalOpen;
end;

procedure TUnStreamRecordSet.InternalOpen;
begin
 ClearCash;
 FCashOfset:=0;
 FCashWriten:=false;
 FCashSize:=FRecordSize*FBufferSize;
 if FCashMode then
  begin
   CopyBufferToRecordMethod:=CopyBufferToRecordBubber;
   CopyRecordToBufferMethod:=CopyRecordToBufferBuffer;
   GetMem(FCash, FCashSize);
   FStream.Position:=FCashOfset;
   if FStream.Size < FCashSize then
    FStream.Size:=FCashSize;
   FStream.Read(FCash^, FCashSize);
  end
 else
  begin
   CopyBufferToRecordMethod:=CopyBufferToRecordWizBubber;
   CopyRecordToBufferMethod:=CopyRecordToBufferWizBuffer;
  end;
end;

procedure TUnStreamRecordSet.OpenFromStream(Str: TStream);
var NewRecData: PUnRecordData;
    CountRec: Integer;
    I: Integer;
begin
 CountRec:=LoadInt(Str);
 FRecordSize:=DataSet.RecordSize+Sizeof(TUnBookmarkInfo);
 FNextOffset:=0;
 Clear;
 for I:=0 to CountRec-1 do
  begin
   NewRecData:=CreateItem;
   FListId.Add(NewRecData);  //     Id
   NewRecData^.Id:=FNextId;
   Inc(FNextId, 1);
   NewRecData^.Position:=FData.Add(NewRecData);  //   
   NewRecData^.BatchState:=ubs_None;
   PUnStrRecordData(NewRecData)^.Offset:=FNextOffset;
   Inc(FNextOffset, FRecordSize);
  end;
 FStream:=Str;
 FStream.Size:=FNextOffset;
 InternalOpen;
end;

function TUnStreamRecordSet.AddData(const Str: TStream): Integer;
var CountRec: Integer;
    BlockSize: Integer;
    I: Integer;
    NewRecData: PUnRecordData; 
begin
 ResetCash;
 CountRec:=LoadInt(Str);
 if CountRec > 0 then
  begin
   BlockSize:=CountRec * FRecordSize;
   FStream.Position:=FNextOffset;
   FStream.CopyFrom(Str, BlockSize);
   for I:=0 to  CountRec-1 do
    begin
     NewRecData:=CreateItem;
     FListId.Add(NewRecData);  //     Id
     NewRecData^.Id:=FNextId;
     Inc(FNextId, 1);
     NewRecData^.Position:=FData.Add(NewRecData);  //   
     NewRecData^.BatchState:=ubs_None;
     PUnStrRecordData(NewRecData)^.Offset:=FNextOffset;
     Inc(FNextOffset, FRecordSize);
    end;
   FStream.Position:=FCashOfset;
   FStream.Read(FCash^, FCashSize);
  end;
 Result:=CountRec;   
end;

procedure TUnStreamRecordSet.SaveData(const Str: TStream);
var OldPosition: Integer;
begin
 ResetCash;
 OldPosition:=FStream.Position;
 try
  SaveInt(FData.Count, Str);
  FStream.Position:=0;
  if FData.Count > 0 then
   Str.CopyFrom(FStream, FNextOffset);             
 finally
  FStream.Position:=OldPosition;
 end;
end;

procedure TUnStreamRecordSet.ResetCash;
begin
 if FCashWriten then
  begin
   FStream.Position:=FCashOfset;
   FStream.Write(FCash^, FCashSize);
   FCashWriten:=false;
  end;
end;

procedure TUnStreamRecordSet.DeleteItem(P: Pointer);
begin
 FreeMem(PUnStrRecordData(P), Sizeof(TUnStrRecordData));
end;

function TUnStreamRecordSet.CreateItem: Pointer;
begin
 GetMem(Result, Sizeof(TUnStrRecordData));
end;

function TUnStreamRecordSet.AddNewItem(Buffer: Pointer; Position: Integer): PUnRecordData;
begin
 Result:=inherited AddNewItem(Buffer, Position);
 PUnStrRecordData(Result)^.Offset:=FNextOffset;
 Inc(FNextOffset, FRecordSize);
 CopyBufferToRecord(Result, Buffer);
end;

procedure TUnStreamRecordSet.CopyRecordToBuffer(Item: Pointer; Buffer: PChar);
begin
 CopyRecordToBufferMethod(PUnStrRecordData(Item)^.Offset, Buffer);
end;
                                    //  bfCurrent, bfBOF, bfEOF, bfInserted
procedure TUnStreamRecordSet.CopyBufferToRecord(Item: Pointer; Buffer: PChar);
begin
 PUnBookmarkInfo(Buffer + DataSet.RecordSize)^.BookmarkFlag:=bfCurrent;
 PUnBookmarkInfo(Buffer + DataSet.RecordSize)^.BookmarkData:=PUnStrRecordData(Item)^.Id;
 CopyBufferToRecordMethod(PUnStrRecordData(Item)^.Offset, Buffer);
end;

//     
procedure TUnStreamRecordSet.CopyBufferToRecordWizBubber(Offset: Integer; Buffer: PChar);
begin  //  
 FStream.Position:=Offset;
 FStream.Write(Buffer^, FRecordSize);
end;

procedure TUnStreamRecordSet.CopyRecordToBufferWizBuffer(Offset: Integer; Buffer: PChar);
begin  //  
 FStream.Position:=Offset;
 FStream.Read(Buffer^, FRecordSize);
end;

procedure TUnStreamRecordSet.CopyBufferToRecordBubber(Offset: Integer; Buffer: PChar);
begin   //  
 CheckFileOffset(Offset);
 Move(Buffer^, (FCash + (Offset - FCashOfset))^, FRecordSize);
 FCashWriten:=true;
end;

procedure TUnStreamRecordSet.CopyRecordToBufferBuffer(Offset: Integer; Buffer: PChar);
begin  //  
 CheckFileOffset(Offset);
 Move((FCash + (Offset - FCashOfset))^, Buffer^, FRecordSize);
end;

//   -   !!!   :-)
procedure TUnStreamRecordSet.CheckFileOffset(AOffset: Longint);
begin // ,      
 if AOffset > (FCashOfset + FCashSize - FRecordSize) then
  begin
   ResetCash;
   FCashOfset:=AOffset;
   if FStream.Size < (FCashOfset + FCashSize) then
    FStream.Size:=FCashOfset + FCashSize;
   FStream.Position:=FCashOfset;
   FStream.Read(FCash^, FCashSize);
  end
 else if AOffset < FCashOfset then
  begin
   ResetCash;
   FCashOfset:=AOffset - FCashSize + FRecordSize;
   if FCashOfset < 0 then
    FCashOfset:=0;
   FStream.Position:=FCashOfset;
   FStream.Read(FCash^, FCashSize);
  end;
end;

function TUnStreamRecordSet.GetStream(AsCopy: boolean): TStream;
var OldPosition: LongInt;
begin
 if AsCopy then
  begin
   Result:=GetTempStream;
   OldPosition:=FStream.Position;
   try
    FStream.Position:=0;
    Result.CopyFrom(FStream, FStream.Size);
   finally
    FStream.Position:=OldPosition;
   end;
  end
 else
  Result:=FStream;
end;

procedure TUnStreamRecordSet.Pack;
var TempStr: TStream;
    I: Integer;
    Rec: PUnStrRecordData;
    Buffer: PChar;
begin
 ResetCash;
 FData.Pack;
 GetMem(Buffer, FRecordSize);
 try
  TempStr:=GetTempStream;
  for I:=0 to FData.Count-1 do
   begin
    Rec:=FData[I];
    FStream.Position:=Rec^.Offset;
    FStream.Read(Buffer^, FRecordSize);
    Rec^.Offset:=TempStr.Position;
    TempStr.Write(Buffer^, FRecordSize);
   end;
  FStream.Free;
  FStream:=TempStr;
  FNextOffset:=FStream.Position;
  CheckFileOffset(FCashOfset);
  FStream.Position:=FCashOfset;
  FStream.Read(FCash^, FCashSize);
  FCashWriten:=false;
 finally
  FreeMem(Buffer, FRecordSize);
 end;
end;

function TUnStreamRecordSet.GetBufferSize: Integer;
begin
 Result:=FRecordSize;
end;

procedure TUnStreamRecordSet.WriteStream(Value: TStream);
begin
 if DataSet.Active then
  raise Exception.Create(ERR_ACTIVE_DATASET);
 FStream:=Value;
end;

procedure TUnStreamRecordSet.WriteBufferSize(Value: Integer);
begin
 if DataSet.Active then
  raise Exception.Create(ERR_ACTIVE_DATASET);
 FBufferSize:=Value;
end;

procedure TUnStreamRecordSet.WriteCashMode(Value: Boolean);
begin
 if DataSet.Active then
  raise Exception.Create(ERR_ACTIVE_DATASET);
 FCashMode:=Value;
end;


{-------------------------TUnStreamRecordSetDispetcher-------------------------}

function TUnStreamRecordSetDispatcher.GetRecordSet(DataSet: TDataSet): TUnCustomRecordSet;
begin
 Result:=TUnStreamRecordSet.Create;
 TUnStreamRecordSet(Result).FStream:=GetTempStream;
 Result.DataSet:=DataSet;
 Result.Capasity:=Capasity;
 TUnStreamRecordSet(Result).FcashMode:=FCashMode;
 TUnStreamRecordSet(Result).Fdispetcher:=Self;
 TUnStreamRecordSet(Result).FBufferSize:=FBufferSize;
end;

end.
